# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import webob

from nova.api.openstack import api_version_request
from nova.api.openstack.compute import remote_consoles \
    as console_v21
from nova.compute import api as compute_api
from nova import exception
from nova import objects
from nova import test
from nova.tests.unit.api.openstack import fakes
from nova.tests.unit import fake_instance


class ConsolesExtensionTestV21(test.NoDBTestCase):
    controller_class = console_v21.RemoteConsolesController
    validation_error = exception.ValidationError

    def setUp(self):
        super(ConsolesExtensionTestV21, self).setUp()
        self.instance = objects.Instance(uuid=fakes.FAKE_UUID)
        self.stub_out('nova.compute.api.API.get',
                      lambda *a, **kw: self.instance)
        self.controller = self.controller_class()

    def _check_console_failure(self, func, expected_exception, body,
                               mocked_method=None, raised_exception=None):
        req = fakes.HTTPRequest.blank('')

        if mocked_method:
            @mock.patch.object(compute_api.API, mocked_method,
                               side_effect=raised_exception)
            def _do_test(mock_method):
                self.assertRaises(expected_exception,
                                  func,
                                  req, fakes.FAKE_UUID, body=body)
                self.assertTrue(mock_method.called)

            _do_test()
        else:
            self.assertRaises(expected_exception,
                              func,
                              req, fakes.FAKE_UUID, body=body)

    @mock.patch.object(compute_api.API, 'get_vnc_console',
                       return_value={'url': 'http://fake'})
    def test_get_vnc_console(self, mock_get_vnc_console):
        body = {'os-getVNCConsole': {'type': 'novnc'}}
        req = fakes.HTTPRequest.blank('')
        output = self.controller.get_vnc_console(req, fakes.FAKE_UUID,
                                                 body=body)
        self.assertEqual(output,
            {u'console': {u'url': u'http://fake', u'type': u'novnc'}})
        mock_get_vnc_console.assert_called_once_with(
            req.environ['nova.context'], self.instance, 'novnc')

    def test_get_vnc_console_not_ready(self):
        body = {'os-getVNCConsole': {'type': 'novnc'}}
        self._check_console_failure(
            self.controller.get_vnc_console,
            webob.exc.HTTPConflict,
            body,
            'get_vnc_console',
            exception.InstanceNotReady(instance_id=fakes.FAKE_UUID))

    def test_get_vnc_console_no_type(self):
        body = {'os-getVNCConsole': {}}
        self._check_console_failure(
            self.controller.get_vnc_console,
            self.validation_error,
            body)

    def test_get_vnc_console_no_instance(self):
        body = {'os-getVNCConsole': {'type': 'novnc'}}
        self._check_console_failure(
            self.controller.get_vnc_console,
            webob.exc.HTTPNotFound,
            body,
            'get',
            exception.InstanceNotFound(instance_id=fakes.FAKE_UUID))

    def test_get_vnc_console_no_instance_on_console_get(self):
        body = {'os-getVNCConsole': {'type': 'novnc'}}
        self._check_console_failure(
            self.controller.get_vnc_console,
            webob.exc.HTTPNotFound,
            body,
            'get_vnc_console',
            exception.InstanceNotFound(instance_id=fakes.FAKE_UUID))

    def test_get_vnc_console_instance_invalid_state(self):
        body = {'os-getVNCConsole': {'type': 'novnc'}}
        self._check_console_failure(
            self.controller.get_vnc_console,
            webob.exc.HTTPConflict,
            body,
            'get_vnc_console',
            exception.InstanceInvalidState(
                attr='fake-attr', state='fake-state', method='fake-method',
                instance_uuid=fakes.FAKE_UUID)
        )

    def test_get_vnc_console_invalid_type(self):
        body = {'os-getVNCConsole': {'type': 'invalid'}}
        self._check_console_failure(
            self.controller.get_vnc_console,
            self.validation_error,
            body)

    def test_get_vnc_console_type_unavailable(self):
        body = {'os-getVNCConsole': {'type': 'unavailable'}}
        self._check_console_failure(
            self.controller.get_vnc_console,
            self.validation_error,
            body)

    def test_get_vnc_console_not_implemented(self):
        body = {'os-getVNCConsole': {'type': 'novnc'}}
        self._check_console_failure(
            self.controller.get_vnc_console,
            webob.exc.HTTPNotImplemented,
            body,
            'get_vnc_console',
            NotImplementedError())

    @mock.patch.object(compute_api.API, 'get_spice_console',
                       return_value={'url': 'http://fake'})
    def test_get_spice_console(self, mock_get_spice_console):
        body = {'os-getSPICEConsole': {'type': 'spice-html5'}}
        req = fakes.HTTPRequest.blank('')
        output = self.controller.get_spice_console(req, fakes.FAKE_UUID,
                                                   body=body)
        self.assertEqual(output,
            {u'console': {u'url': u'http://fake', u'type': u'spice-html5'}})
        mock_get_spice_console.assert_called_once_with(
            req.environ['nova.context'], self.instance, 'spice-html5')

    def test_get_spice_console_not_ready(self):
        body = {'os-getSPICEConsole': {'type': 'spice-html5'}}
        self._check_console_failure(
            self.controller.get_spice_console,
            webob.exc.HTTPConflict,
            body,
            'get_spice_console',
            exception.InstanceNotReady(instance_id=fakes.FAKE_UUID))

    def test_get_spice_console_no_type(self):
        body = {'os-getSPICEConsole': {}}
        self._check_console_failure(
            self.controller.get_spice_console,
            self.validation_error,
            body)

    def test_get_spice_console_no_instance(self):
        body = {'os-getSPICEConsole': {'type': 'spice-html5'}}
        self._check_console_failure(
            self.controller.get_spice_console,
            webob.exc.HTTPNotFound,
            body,
            'get',
            exception.InstanceNotFound(instance_id=fakes.FAKE_UUID))

    def test_get_spice_console_no_instance_on_console_get(self):
        body = {'os-getSPICEConsole': {'type': 'spice-html5'}}
        self._check_console_failure(
            self.controller.get_spice_console,
            webob.exc.HTTPNotFound,
            body,
            'get_spice_console',
            exception.InstanceNotFound(instance_id=fakes.FAKE_UUID))

    def test_get_spice_console_invalid_type(self):
        body = {'os-getSPICEConsole': {'type': 'invalid'}}
        self._check_console_failure(
            self.controller.get_spice_console,
            self.validation_error,
            body)

    def test_get_spice_console_not_implemented(self):
        body = {'os-getSPICEConsole': {'type': 'spice-html5'}}
        self._check_console_failure(
            self.controller.get_spice_console,
            webob.exc.HTTPNotImplemented,
            body,
            'get_spice_console',
            NotImplementedError())

    def test_get_spice_console_type_unavailable(self):
        body = {'os-getSPICEConsole': {'type': 'unavailable'}}
        self._check_console_failure(
            self.controller.get_spice_console,
            self.validation_error,
            body)

    def test_get_rdp_console_bad_request(self):
        body = {'os-getRDPConsole': {'type': 'rdp-html5'}}
        req = fakes.HTTPRequest.blank('')
        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller.get_rdp_console,
                          req, fakes.FAKE_UUID, body=body)

    def test_get_vnc_console_with_undefined_param(self):
        body = {'os-getVNCConsole': {'type': 'novnc', 'undefined': 'foo'}}
        self._check_console_failure(
            self.controller.get_vnc_console,
            self.validation_error,
            body)

    def test_get_spice_console_with_undefined_param(self):
        body = {'os-getSPICEConsole': {'type': 'spice-html5',
                                      'undefined': 'foo'}}
        self._check_console_failure(
            self.controller.get_spice_console,
            self.validation_error,
            body)

    @mock.patch.object(compute_api.API, 'get_serial_console',
                       return_value={'url': 'ws://fake'})
    def test_get_serial_console(self, mock_get_serial_console):
        body = {'os-getSerialConsole': {'type': 'serial'}}
        req = fakes.HTTPRequest.blank('')
        output = self.controller.get_serial_console(req, fakes.FAKE_UUID,
                                                    body=body)
        self.assertEqual({u'console': {u'url': u'ws://fake',
                                       u'type': u'serial'}},
                         output)
        mock_get_serial_console.assert_called_once_with(
            req.environ['nova.context'], self.instance, 'serial')

    def test_get_serial_console_not_enable(self):
        body = {'os-getSerialConsole': {'type': 'serial'}}
        self._check_console_failure(
            self.controller.get_serial_console,
            webob.exc.HTTPBadRequest,
            body,
            'get_serial_console',
            exception.ConsoleTypeUnavailable(console_type="serial"))

    def test_get_serial_console_invalid_type(self):
        body = {'os-getSerialConsole': {'type': 'invalid'}}
        self._check_console_failure(
            self.controller.get_serial_console,
            self.validation_error,
            body)

    def test_get_serial_console_no_type(self):
        body = {'os-getSerialConsole': {}}
        self._check_console_failure(
            self.controller.get_serial_console,
            self.validation_error,
            body)

    def test_get_serial_console_no_instance(self):
        body = {'os-getSerialConsole': {'type': 'serial'}}
        self._check_console_failure(
            self.controller.get_serial_console,
            webob.exc.HTTPNotFound,
            body,
            'get_serial_console',
            exception.InstanceNotFound(instance_id=fakes.FAKE_UUID))

    def test_get_serial_console_instance_not_ready(self):
        body = {'os-getSerialConsole': {'type': 'serial'}}
        self._check_console_failure(
            self.controller.get_serial_console,
            webob.exc.HTTPConflict,
            body,
            'get_serial_console',
            exception.InstanceNotReady(instance_id=fakes.FAKE_UUID))

    def test_get_serial_console_socket_exhausted(self):
        body = {'os-getSerialConsole': {'type': 'serial'}}
        self._check_console_failure(
            self.controller.get_serial_console,
            webob.exc.HTTPBadRequest,
            body,
            'get_serial_console',
            exception.SocketPortRangeExhaustedException(host='127.0.0.1'))

    def test_get_serial_console_image_nport_invalid(self):
        body = {'os-getSerialConsole': {'type': 'serial'}}
        self._check_console_failure(
            self.controller.get_serial_console,
            webob.exc.HTTPBadRequest,
            body,
            'get_serial_console',
            exception.ImageSerialPortNumberInvalid(
                num_ports='x', property="hw_serial_port_count"))

    def test_get_serial_console_image_nport_exceed(self):
        body = {'os-getSerialConsole': {'type': 'serial'}}
        self._check_console_failure(
            self.controller.get_serial_console,
            webob.exc.HTTPBadRequest,
            body,
            'get_serial_console',
            exception.ImageSerialPortNumberExceedFlavorValue())


class ConsolesExtensionTestV26(test.NoDBTestCase):
    def setUp(self):
        super(ConsolesExtensionTestV26, self).setUp()
        self.req = fakes.HTTPRequest.blank('')
        self.context = self.req.environ['nova.context']
        self.req.api_version_request = api_version_request.APIVersionRequest(
            '2.6')
        self.instance = fake_instance.fake_instance_obj(self.context)
        self.stub_out('nova.compute.api.API.get',
                      lambda *a, **kw: self.instance)
        self.controller = console_v21.RemoteConsolesController()

    def test_create_vnc_console(self):
        mock_handler = mock.MagicMock()
        mock_handler.return_value = {'url': "http://fake"}
        self.controller.handlers['vnc'] = mock_handler

        body = {'remote_console': {'protocol': 'vnc', 'type': 'novnc'}}
        output = self.controller.create(self.req, fakes.FAKE_UUID, body=body)
        self.assertEqual({'remote_console': {'protocol': 'vnc',
                                             'type': 'novnc',
                                             'url': 'http://fake'}}, output)
        mock_handler.assert_called_once_with(self.context, self.instance,
                                             'novnc')

    def test_create_spice_console(self):
        mock_handler = mock.MagicMock()
        mock_handler.return_value = {'url': "http://fake"}
        self.controller.handlers['spice'] = mock_handler

        body = {'remote_console': {'protocol': 'spice',
                                   'type': 'spice-html5'}}
        output = self.controller.create(self.req, fakes.FAKE_UUID, body=body)
        self.assertEqual({'remote_console': {'protocol': 'spice',
                                             'type': 'spice-html5',
                                             'url': 'http://fake'}}, output)
        mock_handler.assert_called_once_with(self.context, self.instance,
                                             'spice-html5')

    def test_create_rdp_console_bad_request(self):
        mock_handler = mock.MagicMock()
        mock_handler.return_value = {'url': "http://fake"}
        self.controller.handlers['rdp'] = mock_handler

        body = {'remote_console': {'protocol': 'rdp', 'type': 'rdp-html5'}}
        self.assertRaises(exception.ValidationError, self.controller.create,
                          self.req, fakes.FAKE_UUID, body=body)

    def test_create_serial_console(self):
        mock_handler = mock.MagicMock()
        mock_handler.return_value = {'url': "ws://fake"}
        self.controller.handlers['serial'] = mock_handler

        body = {'remote_console': {'protocol': 'serial', 'type': 'serial'}}
        output = self.controller.create(self.req, fakes.FAKE_UUID, body=body)
        self.assertEqual({'remote_console': {'protocol': 'serial',
                                             'type': 'serial',
                                             'url': 'ws://fake'}}, output)
        mock_handler.assert_called_once_with(self.context, self.instance,
                                             'serial')

    def test_create_console_instance_not_ready(self):
        mock_handler = mock.MagicMock()
        mock_handler.side_effect = exception.InstanceNotReady(
            instance_id='xxx')
        self.controller.handlers['vnc'] = mock_handler

        body = {'remote_console': {'protocol': 'vnc', 'type': 'novnc'}}
        self.assertRaises(webob.exc.HTTPConflict, self.controller.create,
                          self.req, fakes.FAKE_UUID, body=body)

    def test_create_console_unavailable(self):
        mock_handler = mock.MagicMock()
        mock_handler.side_effect = exception.ConsoleTypeUnavailable(
            console_type='vnc')
        self.controller.handlers['vnc'] = mock_handler

        body = {'remote_console': {'protocol': 'vnc', 'type': 'novnc'}}
        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                          self.req, fakes.FAKE_UUID, body=body)
        self.assertTrue(mock_handler.called)

    def test_create_console_not_found(self):
        mock_handler = mock.MagicMock()
        mock_handler.side_effect = exception.InstanceNotFound(
            instance_id='xxx')
        self.controller.handlers['vnc'] = mock_handler

        body = {'remote_console': {'protocol': 'vnc', 'type': 'novnc'}}
        self.assertRaises(webob.exc.HTTPNotFound, self.controller.create,
                          self.req, fakes.FAKE_UUID, body=body)

    def test_create_console_not_implemented(self):
        mock_handler = mock.MagicMock()
        mock_handler.side_effect = NotImplementedError()
        self.controller.handlers['vnc'] = mock_handler

        body = {'remote_console': {'protocol': 'vnc', 'type': 'novnc'}}
        self.assertRaises(webob.exc.HTTPNotImplemented, self.controller.create,
                          self.req, fakes.FAKE_UUID, body=body)

    def test_create_console_nport_invalid(self):
        mock_handler = mock.MagicMock()
        mock_handler.side_effect = exception.ImageSerialPortNumberInvalid(
            num_ports='x', property="hw_serial_port_count")
        self.controller.handlers['serial'] = mock_handler

        body = {'remote_console': {'protocol': 'serial', 'type': 'serial'}}
        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                          self.req, fakes.FAKE_UUID, body=body)

    def test_create_console_nport_exceed(self):
        mock_handler = mock.MagicMock()
        mock_handler.side_effect = (
            exception.ImageSerialPortNumberExceedFlavorValue())
        self.controller.handlers['serial'] = mock_handler

        body = {'remote_console': {'protocol': 'serial', 'type': 'serial'}}
        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                          self.req, fakes.FAKE_UUID, body=body)

    def test_create_console_socket_exhausted(self):
        mock_handler = mock.MagicMock()
        mock_handler.side_effect = (
            exception.SocketPortRangeExhaustedException(host='127.0.0.1'))
        self.controller.handlers['serial'] = mock_handler

        body = {'remote_console': {'protocol': 'serial', 'type': 'serial'}}
        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                          self.req, fakes.FAKE_UUID, body=body)

    def test_create_console_invalid_type(self):
        mock_handler = mock.MagicMock()
        mock_handler.side_effect = (
            exception.ConsoleTypeInvalid(console_type='invalid_type'))
        self.controller.handlers['serial'] = mock_handler
        body = {'remote_console': {'protocol': 'serial', 'type': 'xvpvnc'}}
        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                          self.req, fakes.FAKE_UUID, body=body)


class ConsolesExtensionTestV28(ConsolesExtensionTestV26):
    def setUp(self):
        super(ConsolesExtensionTestV28, self).setUp()
        self.req = fakes.HTTPRequest.blank('')
        self.context = self.req.environ['nova.context']
        self.req.api_version_request = api_version_request.APIVersionRequest(
            '2.8')
        self.controller = console_v21.RemoteConsolesController()

    def test_create_mks_console(self):
        mock_handler = mock.MagicMock()
        mock_handler.return_value = {'url': "http://fake"}
        self.controller.handlers['mks'] = mock_handler

        body = {'remote_console': {'protocol': 'mks', 'type': 'webmks'}}
        output = self.controller.create(self.req, fakes.FAKE_UUID, body=body)
        self.assertEqual({'remote_console': {'protocol': 'mks',
                                             'type': 'webmks',
                                             'url': 'http://fake'}}, output)
        mock_handler.assert_called_once_with(self.context, self.instance,
                                             'webmks')



# Copyright (c) 2013 ISP RAS.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from nova.scheduler import filters

LOG = logging.getLogger(__name__)


class PciPassthroughFilter(
    filters.BaseHostFilter,
    filters.CandidateFilterMixin,
):
    """Pci Passthrough Filter based on PCI request

    Filter that schedules instances on a host if the host has devices
    to meet the device requests in the 'extra_specs' for the flavor.

    PCI resource tracker provides updated summary information about the
    PCI devices for each host, like::

        | [{"count": 5, "vendor_id": "8086", "product_id": "1520",
        |   "extra_info":'{}'}],

    and VM requests PCI devices via PCI requests, like::

        | [{"count": 1, "vendor_id": "8086", "product_id": "1520",}].

    The filter checks if the host passes or not based on this information.

    """

    RUN_ON_REBUILD = False

    def host_passes(self, host_state, spec_obj):
        """Return true if the host has the required PCI devices."""
        pci_requests = spec_obj.pci_requests
        if not pci_requests or not pci_requests.requests:
            return True

        if not host_state.pci_stats:
            LOG.debug("%(host_state)s doesn't have the required PCI devices"
                      " (%(requests)s)",
                      {'host_state': host_state, 'requests': pci_requests})
            return False

        good_candidates = self.filter_candidates(
            host_state,
            lambda candidate: host_state.pci_stats.support_requests(
                pci_requests.requests, provider_mapping=candidate["mappings"]
            ),
        )

        if not good_candidates:
            LOG.debug("%(host_state)s doesn't have the required PCI devices"
                      " (%(requests)s)",
                      {'host_state': host_state, 'requests': pci_requests})
            return False

        return True



# Copyright 2011 Eldar Nugaev
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_serialization import jsonutils
from oslo_utils.fixture import uuidsentinel as uuids

from nova.api.openstack import wsgi as os_wsgi
from nova.compute import api as compute_api
from nova import exception
from nova import objects
from nova import test
from nova.tests.unit.api.openstack import fakes


UUID = uuids.abc


def fake_instance_get(self, _context, instance_uuid, expected_attrs=None,
                      cell_down_support=False):
    if instance_uuid != UUID:
        raise Exception("Invalid UUID")
    return objects.Instance(uuid=instance_uuid,
                            project_id=_context.project_id,
                            host='123')


class ServerDiagnosticsTestV21(test.NoDBTestCase):
    mock_diagnostics_method = 'get_diagnostics'
    api_version = '2.1'

    def _setup_router(self):
        self.router = fakes.wsgi_app_v21()

    def _get_request(self):
        return fakes.HTTPRequest.blank(
            '/v2/%s/servers/%s/diagnostics' % (fakes.FAKE_PROJECT_ID, UUID),
            version=self.api_version,
            headers = {os_wsgi.API_VERSION_REQUEST_HEADER:
                           'compute %s' % self.api_version})

    def setUp(self):
        super(ServerDiagnosticsTestV21, self).setUp()
        self._setup_router()

    @mock.patch.object(compute_api.API, 'get', fake_instance_get)
    def _test_get_diagnostics(self, expected, return_value):
        req = self._get_request()
        with mock.patch.object(compute_api.API, self.mock_diagnostics_method,
                               return_value=return_value):
            res = req.get_response(self.router)
        output = jsonutils.loads(res.body)
        self.assertEqual(expected, output)

    def test_get_diagnostics(self):
        diagnostics = {'data': 'Some diagnostics info'}
        self._test_get_diagnostics(diagnostics, diagnostics)

    @mock.patch.object(compute_api.API, 'get',
                side_effect=exception.InstanceNotFound(instance_id=UUID))
    def test_get_diagnostics_with_non_existed_instance(self, mock_get):
        req = self._get_request()
        res = req.get_response(self.router)
        self.assertEqual(res.status_int, 404)

    @mock.patch.object(compute_api.API, 'get', fake_instance_get)
    def test_get_diagnostics_raise_conflict_on_invalid_state(self):
        req = self._get_request()
        with mock.patch.object(compute_api.API, self.mock_diagnostics_method,
                side_effect=exception.InstanceInvalidState('fake message')):
            res = req.get_response(self.router)
        self.assertEqual(409, res.status_int)

    @mock.patch.object(compute_api.API, 'get', fake_instance_get)
    def test_get_diagnostics_raise_instance_not_ready(self):
        req = self._get_request()
        with mock.patch.object(compute_api.API, self.mock_diagnostics_method,
                side_effect=exception.InstanceNotReady('fake message')):
            res = req.get_response(self.router)
        self.assertEqual(409, res.status_int)

    @mock.patch.object(compute_api.API, 'get', fake_instance_get)
    def test_get_diagnostics_raise_no_notimplementederror(self):
        req = self._get_request()
        with mock.patch.object(compute_api.API, self.mock_diagnostics_method,
                               side_effect=NotImplementedError):
            res = req.get_response(self.router)
        self.assertEqual(501, res.status_int)


class ServerDiagnosticsTestV248(ServerDiagnosticsTestV21):
    mock_diagnostics_method = 'get_instance_diagnostics'
    api_version = '2.48'

    def test_get_diagnostics(self):
        return_value = objects.Diagnostics(
            config_drive=False,
            state='running',
            driver='libvirt',
            uptime=5,
            hypervisor='hypervisor',
            # hypervisor_os is unset
            cpu_details=[
                objects.CpuDiagnostics(id=0, time=1111, utilisation=11),
                objects.CpuDiagnostics(id=1, time=None, utilisation=22),
                objects.CpuDiagnostics(id=2, time=3333, utilisation=None),
                objects.CpuDiagnostics(id=None, time=4444, utilisation=44)],
            nic_details=[objects.NicDiagnostics(
                mac_address='de:ad:be:ef:00:01',
                rx_drop=1,
                rx_errors=2,
                rx_octets=3,
                rx_packets=4,
                rx_rate=5,
                tx_drop=6,
                tx_errors=7,
                tx_octets=8,
                # tx_packets is unset
                tx_rate=None)],
            disk_details=[objects.DiskDiagnostics(
                errors_count=1,
                read_bytes=2,
                read_requests=3,
                # write_bytes is unset
                write_requests=None)],
            num_cpus=4,
            num_disks=1,
            num_nics=1,
            memory_details=objects.MemoryDiagnostics(maximum=8192, used=3072))

        expected = {
            'config_drive': False,
            'state': 'running',
            'driver': 'libvirt',
            'uptime': 5,
            'hypervisor': 'hypervisor',
            'hypervisor_os': None,
            'cpu_details': [{'id': 0, 'time': 1111, 'utilisation': 11},
                            {'id': 1, 'time': None, 'utilisation': 22},
                            {'id': 2, 'time': 3333, 'utilisation': None},
                            {'id': None, 'time': 4444, 'utilisation': 44}],
            'nic_details': [{'mac_address': 'de:ad:be:ef:00:01',
                             'rx_drop': 1,
                             'rx_errors': 2,
                             'rx_octets': 3,
                             'rx_packets': 4,
                             'rx_rate': 5,
                             'tx_drop': 6,
                             'tx_errors': 7,
                             'tx_octets': 8,
                             'tx_packets': None,
                             'tx_rate': None}],
            'disk_details': [{'errors_count': 1,
                              'read_bytes': 2,
                              'read_requests': 3,
                              'write_bytes': None,
                              'write_requests': None}],
            'num_cpus': 4,
            'num_disks': 1,
            'num_nics': 1,
            'memory_details': {'maximum': 8192, 'used': 3072}}

        self._test_get_diagnostics(expected, return_value)



# Copyright (c) 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures


from nova.compute import resource_tracker


class FakeResourceTracker(resource_tracker.ResourceTracker):
    """Version without a DB requirement."""

    def _update(self, context, compute_node, startup=False):
        pass


class RTMockMixin(object):
    def _mock_rt(self, **kwargs):
        if 'spec_set' in kwargs:
            kwargs.update({'autospec': False})
        return self.useFixture(fixtures.MockPatchObject(
            self.compute, 'rt', **kwargs)).mock



# Copyright 2020 Red Hat, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Validators for ``pci_passthrough`` namespaced extra specs."""

from nova.api.validation.extra_specs import base


EXTRA_SPEC_VALIDATORS = [
    base.ExtraSpecValidator(
        name='pci_passthrough:alias',
        description=(
            'Specify the number of ``$alias`` PCI device(s) to attach to the '
            'instance. '
            'Must be of format ``$alias:$count``, where ``$alias`` '
            'corresponds to a particular PCI device class (as configured in '
            '``nova.conf``) and ``$count`` is the amount of PCI devices of '
            'type ``$alias`` to be assigned to the instance. '
            'Use commas to specify multiple values. '
            'Only supported by the libvirt virt driver.'
        ),
        value={
            'type': str,
            # one or more comma-separated '$alias:$count' values
            'pattern': r'[^:]+:\d+(?:\s*,\s*[^:]+:\d+)*',
        },
    ),
]


def register():
    return EXTRA_SPEC_VALIDATORS



# Copyright (c) 2012 NTT DOCOMO, INC.
# Copyright 2010 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Mapping of bare metal node states.

Setting the node `power_state` is handled by the conductor's power
synchronization thread. Based on the power state retrieved from the driver
for the node, the state is set to POWER_ON or POWER_OFF, accordingly.
Should this fail, the `power_state` value is left unchanged, and the node
is placed into maintenance mode.

The `power_state` can also be set manually via the API. A failure to change
the state leaves the current state unchanged. The node is NOT placed into
maintenance mode in this case.
"""


#####################
# Provisioning states
#####################

NOSTATE = None
""" No state information.

This state is used with power_state to represent a lack of knowledge of
power state, and in target_*_state fields when there is no target.

Prior to the Kilo release, Ironic set node.provision_state to NOSTATE
when the node was available for provisioning. During Kilo cycle, this was
changed to the AVAILABLE state.
"""

MANAGEABLE = 'manageable'
""" Node is in a manageable state.
This state indicates that Ironic has verified, at least once, that it had
sufficient information to manage the hardware. While in this state, the node
is not available for provisioning (it must be in the AVAILABLE state for that).
"""

AVAILABLE = 'available'
""" Node is available for use and scheduling.

This state is replacing the NOSTATE state used prior to Kilo.
"""

ACTIVE = 'active'
""" Node is successfully deployed and associated with an instance. """

DEPLOYWAIT = 'wait call-back'
""" Node is waiting to be deployed.

This will be the node `provision_state` while the node is waiting for
the driver to finish deployment.
"""

DEPLOYING = 'deploying'
""" Node is ready to receive a deploy request, or is currently being deployed.

A node will have its `provision_state` set to DEPLOYING briefly before it
receives its initial deploy request. It will also move to this state from
DEPLOYWAIT after the callback is triggered and deployment is continued
(disk partitioning and image copying).
"""

DEPLOYFAIL = 'deploy failed'
""" Node deployment failed. """

DEPLOYDONE = 'deploy complete'
""" Node was successfully deployed.

This is mainly a target provision state used during deployment. A successfully
deployed node should go to ACTIVE status.
"""

DELETING = 'deleting'
""" Node is actively being torn down. """

DELETED = 'deleted'
""" Node tear down was successful.

In Juno, target_provision_state was set to this value during node tear down.
In Kilo, this will be a transitory value of provision_state, and never
represented in target_provision_state.
"""

CLEANING = 'cleaning'
""" Node is being automatically cleaned to prepare it for provisioning. """

CLEANWAIT = 'clean wait'
""" Node is waiting for a clean step to be finished.

This will be the node's `provision_state` while the node is waiting for
the driver to finish a cleaning step.
"""

CLEANFAIL = 'clean failed'
""" Node failed cleaning. This requires operator intervention to resolve. """

ERROR = 'error'
""" An error occurred during node processing.

The `last_error` attribute of the node details should contain an error message.
"""

REBUILD = 'rebuild'
""" Node is to be rebuilt.
This is not used as a state, but rather as a "verb" when changing the node's
provision_state via the REST API.
"""

INSPECTING = 'inspecting'
""" Node is under inspection.
This is the provision state used when inspection is started. A successfully
inspected node shall transition to MANAGEABLE status.
"""

INSPECTFAIL = 'inspect failed'
""" Node inspection failed. """


RESCUE = 'rescue'
""" Node is in rescue mode.
This is also used as a "verb" when changing the node's provision_state via the
REST API"""

RESCUEFAIL = 'rescue failed'
""" Node rescue failed. """

RESCUEWAIT = 'rescue wait'
""" Node is waiting for rescue callback. """

RESCUING = 'rescuing'
""" Node is waiting to be rescued. """

UNRESCUE = 'unrescue'
""" Node is to be unrescued.
This is not used as a state, but rather as a "verb" when changing the node's
provision_state via the REST API.
"""

UNRESCUEFAIL = 'unrescue failed'
""" Node unrescue failed. """

UNRESCUING = "unrescuing"
""" Node is unrescuing. """

##############
# Power states
##############

POWER_ON = 'power on'
""" Node is powered on. """

POWER_OFF = 'power off'
""" Node is powered off. """

REBOOT = 'rebooting'
""" Node is rebooting. """

##################
# Helper constants
##################

PROVISION_STATE_LIST = (NOSTATE, MANAGEABLE, AVAILABLE, ACTIVE, DEPLOYWAIT,
                        DEPLOYING, DEPLOYFAIL, DEPLOYDONE, DELETING, DELETED,
                        CLEANING, CLEANWAIT, CLEANFAIL, ERROR, REBUILD,
                        INSPECTING, INSPECTFAIL)
""" A list of all provision states. """



# Copyright 2013 UnitedStack Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Port fields

BINDING_PROFILE = 'binding:profile'
BINDING_HOST_ID = 'binding:host_id'
RESOURCE_REQUEST = 'resource_request'
REQUEST_GROUPS = 'request_groups'
NUMA_POLICY = 'numa_affinity_policy'

# Binding profile fields

MIGRATING_ATTR = 'migrating_to'
ALLOCATION = 'allocation'

# Core extensions

DNS_INTEGRATION = 'dns-integration'
MULTI_PROVIDER = 'multi-provider'
FIP_PORT_DETAILS = 'fip-port-details'
PORT_BINDING = 'binding'
PORT_BINDING_EXTENDED = 'binding-extended'
SUBSTR_PORT_FILTERING = 'ip-substring-filtering'
SEGMENT = 'segment'
RESOURCE_REQUEST_GROUPS = 'port-resource-request-groups'

# Third-party extensions

VNIC_INDEX = 'vnic-index'  # this is provided by the vmware_nsx project

# Search fields

NET_EXTERNAL = 'router:external'

# Misc

DEFAULT_SECGROUP = 'default'
L3_NETWORK_TYPES = ['vxlan', 'gre', 'geneve']



# Copyright (C) 2018 Red Hat, Inc
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import io
from unittest import mock

import fixtures
from oslo_utils.fixture import uuidsentinel as uuids

from nova import conf
from nova.tests import fixtures as nova_fixtures
from nova.tests.fixtures import libvirt as fakelibvirt
from nova.tests.functional import integrated_helpers


CONF = conf.CONF


class ServersTestBase(integrated_helpers._IntegratedTestBase):
    """A libvirt-specific variant of the integrated test base."""

    ADDITIONAL_FILTERS = []

    def setUp(self):
        self.flags(instances_path=self.useFixture(fixtures.TempDir()).path)

        self.computes = {}
        self.compute_rp_uuids = {}

        super(ServersTestBase, self).setUp()

        self.useFixture(nova_fixtures.LibvirtImageBackendFixture())
        self.useFixture(nova_fixtures.CGroupsFixture())
        self.libvirt = self.useFixture(nova_fixtures.LibvirtFixture())
        self.useFixture(nova_fixtures.OSBrickFixture())

        self.useFixture(fixtures.MockPatch(
            'nova.virt.libvirt.LibvirtDriver._create_image',
            return_value=(False, False)))
        self.useFixture(fixtures.MockPatch(
            'nova.virt.libvirt.LibvirtDriver._get_local_gb_info',
            return_value={'total': 128, 'used': 44, 'free': 84}))
        self.mock_is_valid_hostname = self.useFixture(fixtures.MockPatch(
            'nova.virt.libvirt.driver.libvirt_utils.is_valid_hostname',
            return_value=True)).mock
        self.mock_file_open = self.useFixture(fixtures.MockPatch(
            'nova.virt.libvirt.driver.libvirt_utils.file_open',
            side_effect=lambda *a, **k: io.BytesIO(b''))).mock
        self.useFixture(fixtures.MockPatch(
            'nova.privsep.utils.supports_direct_io',
            return_value=True))
        self.useFixture(fixtures.MockPatch(
            'nova.virt.libvirt.host.Host.get_online_cpus',
            return_value=set(range(16))))

        # Mock the 'get_connection' function, as we're going to need to provide
        # custom capabilities for each test
        _p = mock.patch('nova.virt.libvirt.host.Host.get_connection')
        self.mock_conn = _p.start()
        self.addCleanup(_p.stop)

    def _setup_compute_service(self):
        # NOTE(stephenfin): We don't start the compute service here as we wish
        # to configure the host capabilities first. We instead start the
        # service in the test
        self.flags(compute_driver='libvirt.LibvirtDriver')

    def _setup_scheduler_service(self):
        enabled_filters = CONF.filter_scheduler.enabled_filters
        enabled_filters += self.ADDITIONAL_FILTERS

        self.flags(enabled_filters=enabled_filters, group='filter_scheduler')

        return self.start_service('scheduler')

    def _get_connection(
        self, host_info=None, pci_info=None, mdev_info=None, vdpa_info=None,
        libvirt_version=None, qemu_version=None, hostname=None,
    ):
        if not host_info:
            host_info = fakelibvirt.HostInfo(
                cpu_nodes=2, cpu_sockets=1, cpu_cores=2, cpu_threads=2)

        # sanity check
        self.assertGreater(16, host_info.cpus,
            "Host.get_online_cpus is only accounting for 16 CPUs but you're "
            "requesting %d; change the mock or your test" % host_info.cpus)

        libvirt_version = libvirt_version or fakelibvirt.FAKE_LIBVIRT_VERSION
        qemu_version = qemu_version or fakelibvirt.FAKE_QEMU_VERSION

        fake_connection = fakelibvirt.Connection(
            'qemu:///system',
            version=libvirt_version,
            hv_version=qemu_version,
            host_info=host_info,
            pci_info=pci_info,
            mdev_info=mdev_info,
            vdpa_info=vdpa_info,
            hostname=hostname)
        return fake_connection

    def start_compute(
        self, hostname='compute1', host_info=None, pci_info=None,
        mdev_info=None, vdpa_info=None, libvirt_version=None,
        qemu_version=None, cell_name=None, connection=None
    ):
        """Start a compute service.

        The started service will be saved in self.computes, keyed by hostname.

        :param hostname: A hostname.
        :param host_info: A fakelibvirt.HostInfo object for the host. Defaults
            to a HostInfo with 2 NUMA nodes, 2 cores per node, 2 threads per
            core, and 16GB of RAM.
        :param connection: A fake libvirt connection. You should not provide it
            directly. However it is used by restart_compute_service to
            implement restart without losing the hypervisor state.
        :returns: The hostname of the created service, which can be used to
            lookup the created service and UUID of the associated resource
            provider.
        """
        if connection and (
            host_info or
            pci_info or
            mdev_info or
            vdpa_info or
            libvirt_version or
            qemu_version
        ):
            raise ValueError(
                "Either an existing connection instance can be provided or a "
                "list of parameters for a new connection"
            )

        def _start_compute(hostname, host_info):
            if connection:
                fake_connection = connection
            else:
                fake_connection = self._get_connection(
                    host_info, pci_info, mdev_info, vdpa_info, libvirt_version,
                    qemu_version, hostname,
                )

            # If the compute is configured with PCI devices then we need to
            # make sure that the stubs around sysfs has the MAC address
            # information for the PCI PF devices
            if pci_info:
                self.libvirt.update_sriov_mac_address_mapping(
                    pci_info.get_pci_address_mac_mapping())
            # This is fun. Firstly we need to do a global'ish mock so we can
            # actually start the service.
            orig_con = self.mock_conn.return_value
            self.mock_conn.return_value = fake_connection
            compute = self.start_service(
                'compute', host=hostname, cell_name=cell_name)
            # Once that's done, we need to tweak the compute "service" to
            # make sure it returns unique objects.
            compute.driver._host.get_connection = lambda: fake_connection
            # Then we revert the local mock tweaking so the next compute can
            # get its own
            self.mock_conn.return_value = orig_con
            return compute

        # ensure we haven't already registered services with these hostnames
        self.assertNotIn(hostname, self.computes)
        self.assertNotIn(hostname, self.compute_rp_uuids)

        with mock.patch('nova.virt.node.get_local_node_uuid') as m:
            m.return_value = str(getattr(uuids, 'node_%s' % hostname))
            self.computes[hostname] = _start_compute(hostname, host_info)
            # We need to trigger libvirt.Host() to capture the node-local
            # uuid while we have it mocked out.
            self.computes[hostname].driver._host.get_node_uuid()

        self.compute_rp_uuids[hostname] = self.placement.get(
            '/resource_providers?name=%s' % hostname).body[
            'resource_providers'][0]['uuid']

        return hostname

    def restart_compute_service(
        self,
        hostname,
        host_info=None,
        pci_info=None,
        mdev_info=None,
        vdpa_info=None,
        libvirt_version=None,
        qemu_version=None,
        keep_hypervisor_state=True,
    ):
        """Stops the service and starts a new one to have realistic restart

        :param hostname: the hostname of the nova-compute service to be
            restarted
        :param keep_hypervisor_state: If True then we reuse the fake connection
            from the existing driver. If False a new connection will be created
            based on the other parameters provided
        """
        # We are intentionally not calling super() here. Nova's base test class
        # defines starting and restarting compute service with a very
        # different signatures and also those calls are cannot be made aware of
        # the intricacies of the libvirt fixture. So we simply hide that
        # implementation.

        if keep_hypervisor_state and (
            host_info or
            pci_info or
            mdev_info or
            vdpa_info or
            libvirt_version or
            qemu_version
        ):
            raise ValueError(
                "Either keep_hypervisor_state=True or a list of libvirt "
                "parameters can be provided but not both"
            )

        compute = self.computes.pop(hostname)
        self.compute_rp_uuids.pop(hostname)

        # NOTE(gibi): The service interface cannot be used to simulate a real
        # service restart as the manager object will not be recreated after a
        # service.stop() and service.start() therefore the manager state will
        # survive. For example the resource tracker will not be recreated after
        # a stop start. The service.kill() call cannot help as it deletes
        # the service from the DB which is unrealistic and causes that some
        # operation that refers to the killed host (e.g. evacuate) fails.
        # So this helper method will stop the original service and then starts
        # a brand new compute service for the same host and node. This way
        # a new ComputeManager instance will be created and initialized during
        # the service startup.
        compute.stop()

        # this service was running previously, so we have to make sure that
        # we restart it in the same cell
        cell_name = self.host_mappings[compute.host].cell_mapping.name

        old_connection = compute.manager.driver._get_connection()

        self.start_compute(
            hostname, host_info, pci_info, mdev_info, vdpa_info,
            libvirt_version, qemu_version, cell_name,
            old_connection if keep_hypervisor_state else None
        )

        return self.computes[hostname]


class LibvirtMigrationMixin(object):
    """A simple mixin to facilliate successful libvirt live migrations

    Requires that the test class set self.server for the specific test instance
    and self.{src,dest} to indicate the direction of the migration. For any
    scenarios more complex than this they should override _migrate_stub with
    their own implementation.
    """

    def setUp(self):
        super().setUp()
        self.useFixture(fixtures.MonkeyPatch(
            'nova.tests.fixtures.libvirt.Domain.migrateToURI3',
            self._migrate_stub))
        self.migrate_stub_ran = False

    def _migrate_stub(self, domain, destination, params, flags):
        self.dest.driver._host.get_connection().createXML(
            params['destination_xml'],
            'fake-createXML-doesnt-care-about-flags')
        conn = self.src.driver._host.get_connection()
        dom = conn.lookupByUUIDString(self.server['id'])
        dom.complete_job()
        self.migrate_stub_ran = True


class LibvirtNeutronFixture(nova_fixtures.NeutronFixture):
    """A custom variant of the stock neutron fixture with more networks.

    There are three networks available: two l2 networks (one flat and one VLAN)
    and one l3 network (VXLAN).
    """
    network_1 = {
        'id': '3cb9bc59-5699-4588-a4b1-b87f96708bc6',
        'status': 'ACTIVE',
        'subnets': [],
        'name': 'physical-network-foo',
        'admin_state_up': True,
        'tenant_id': nova_fixtures.NeutronFixture.tenant_id,
        'provider:physical_network': 'foo',
        'provider:network_type': 'flat',
        'provider:segmentation_id': None,
    }
    network_2 = network_1.copy()
    network_2.update({
        'id': 'a252b8cd-2d99-4e82-9a97-ec1217c496f5',
        'name': 'physical-network-bar',
        'provider:physical_network': 'bar',
        'provider:network_type': 'vlan',
        'provider:segmentation_id': 123,
    })
    network_3 = network_1.copy()
    network_3.update({
        'id': '877a79cc-295b-4b80-9606-092bf132931e',
        'name': 'tunneled-network',
        'provider:physical_network': None,
        'provider:network_type': 'vxlan',
        'provider:segmentation_id': 69,
    })
    network_4 = network_1.copy()
    network_4.update({
        'id': '1b70879f-fd00-411e-8ea9-143e7820e61d',
        'name': 'private-network',
        'shared': False,
        'provider:physical_network': 'physnet4',
        "provider:network_type": "vlan",
        'provider:segmentation_id': 42,
    })

    subnet_1 = nova_fixtures.NeutronFixture.subnet_1.copy()
    subnet_1.update({
        'name': 'physical-subnet-foo',
    })
    subnet_2 = nova_fixtures.NeutronFixture.subnet_1.copy()
    subnet_2.update({
        'id': 'b4c13749-c002-47ed-bf42-8b1d44fa9ff2',
        'name': 'physical-subnet-bar',
        'network_id': network_2['id'],
    })
    subnet_3 = nova_fixtures.NeutronFixture.subnet_1.copy()
    subnet_3.update({
        'id': '4dacb20b-917f-4275-aa75-825894553442',
        'name': 'tunneled-subnet',
        'network_id': network_3['id'],
    })
    subnet_4 = nova_fixtures.NeutronFixture.subnet_1.copy()
    subnet_4.update({
        'id': '7cb343ec-6637-494c-89a1-8890eab7788e',
        'name': 'physical-subnet-bar',
        'network_id': network_4['id'],
    })
    network_1['subnets'] = [subnet_1]
    network_2['subnets'] = [subnet_2]
    network_3['subnets'] = [subnet_3]
    network_4['subnets'] = [subnet_4]

    network_1_port_2 = {
        'id': 'f32582b5-8694-4be8-9a52-c5732f601c9d',
        'network_id': network_1['id'],
        'status': 'ACTIVE',
        'mac_address': '71:ce:c7:8b:cd:dc',
        'fixed_ips': [
            {
                'ip_address': '192.168.1.10',
                'subnet_id': subnet_1['id']
            }
        ],
        'binding:vif_details': {},
        'binding:vif_type': 'ovs',
        'binding:vnic_type': 'normal',
    }
    network_1_port_3 = {
        'id': '9c7580a0-8b01-41f3-ba07-a114709a4b74',
        'network_id': network_1['id'],
        'status': 'ACTIVE',
        'mac_address': '71:ce:c7:2b:cd:dc',
        'fixed_ips': [
            {
                'ip_address': '192.168.1.11',
                'subnet_id': subnet_1['id']
            }
        ],
        'binding:vif_details': {},
        'binding:vif_type': 'ovs',
        'binding:vnic_type': 'normal',
    }
    network_2_port_1 = {
        'id': '67d36444-6353-40f5-9e92-59346cf0dfda',
        'network_id': network_2['id'],
        'status': 'ACTIVE',
        'mac_address': 'd2:0b:fd:d7:89:9b',
        'fixed_ips': [
            {
                'ip_address': '192.168.1.6',
                'subnet_id': subnet_2['id']
            }
        ],
        'binding:vif_details': {},
        'binding:vif_type': 'ovs',
        'binding:vnic_type': 'normal',
    }
    network_3_port_1 = {
        'id': '4bfa1dc4-4354-4840-b0b4-f06196fa1344',
        'network_id': network_3['id'],
        'status': 'ACTIVE',
        'mac_address': 'd2:0b:fd:99:89:9b',
        'fixed_ips': [
            {
                'ip_address': '192.168.2.6',
                'subnet_id': subnet_3['id']
            }
        ],
        'binding:vif_details': {},
        'binding:vif_type': 'ovs',
        'binding:vnic_type': 'normal',
    }
    network_3_port_2 = {
        'id': '132c3875-b175-4b20-8a57-7a76219a13ae',
        'network_id': network_3['id'],
        'status': 'ACTIVE',
        'mac_address': 'd2:0b:fd:99:89:8b',
        'fixed_ips': [
            {
                'ip_address': '192.168.2.7',
                'subnet_id': subnet_3['id']
            }
        ],
        'binding:vif_details': {},
        'binding:vif_type': 'ovs',
        'binding:vnic_type': 'remote-managed',
    }
    network_4_port_1 = {
        'id': 'b4cd0b93-2ac8-40a7-9fa4-2cd680ccdf3e',
        'network_id': network_4['id'],
        'status': 'ACTIVE',
        'mac_address': 'b5:bc:2e:e7:51:ee',
        'fixed_ips': [
            {
                'ip_address': '192.168.4.6',
                'subnet_id': subnet_4['id']
            }
        ],
        'binding:vif_details': {'vlan': 42},
        'binding:vif_type': 'hw_veb',
        'binding:vnic_type': 'direct',
    }
    network_4_port_2 = {
        'id': '4a0e3b05-4704-4adb-bfb1-f31f0e4d1bdc',
        'network_id': network_4['id'],
        'status': 'ACTIVE',
        'mac_address': 'b5:bc:2e:e7:51:ef',
        'fixed_ips': [
            {
                'ip_address': '192.168.4.7',
                'subnet_id': subnet_4['id']
            }
        ],
        'binding:vif_details': {'vlan': 42},
        'binding:vif_type': 'hw_veb',
        'binding:vnic_type': 'direct',
    }
    network_4_port_3 = {
        'id': 'fb2de1a1-d096-41be-9dbe-43066da64804',
        'network_id': network_4['id'],
        'status': 'ACTIVE',
        'mac_address': 'b5:bc:2e:e7:51:ff',
        'fixed_ips': [
            {
                'ip_address': '192.168.4.8',
                'subnet_id': subnet_4['id']
            }
        ],
        'binding:vif_details': {'vlan': 42},
        'binding:vif_type': 'hw_veb',
        'binding:vnic_type': 'direct',
    }
    network_4_port_4 = {
        'id': 'a31e381d-41ec-41e4-b5a5-ec4ef705fafa',
        'network_id': network_1['id'],
        'status': 'ACTIVE',
        'mac_address': '71:ce:c7:2b:cd:dd',
        'fixed_ips': [
            {
                'ip_address': '192.168.4.9',
                'subnet_id': subnet_4['id']
            }
        ],
        'binding:vif_details': {},
        'binding:vif_type': 'ovs',
        'binding:vnic_type': 'remote-managed',
    }

    network_4_port_pf = {
        'id': 'c6f51315-9202-416f-9e2f-eb78b3ac36d9',
        'network_id': network_4['id'],
        'status': 'ACTIVE',
        'mac_address': 'b5:bc:2e:e7:51:01',
        'fixed_ips': [
            {
                'ip_address': '192.168.4.8',
                'subnet_id': subnet_4['id']
            }
        ],
        'binding:vif_details': {'vlan': 42},
        'binding:vif_type': 'hostdev_physical',
        'binding:vnic_type': 'direct-physical',
    }

    def __init__(self, test):
        super(LibvirtNeutronFixture, self).__init__(test)
        self._networks = {
            self.network_1['id']: self.network_1,
            self.network_2['id']: self.network_2,
            self.network_3['id']: self.network_3,
            self.network_4['id']: self.network_4,
        }
        self._net1_ports = [self.network_1_port_2, self.network_1_port_3]

    def create_port(self, body=None):
        network_id = body['port']['network_id']
        assert network_id in self._networks, ('Network %s not in fixture' %
                                              network_id)

        if network_id == self.network_1['id']:
            port = self._net1_ports.pop(0)
        elif network_id == self.network_2['id']:
            port = self.network_2_port_1
        elif network_id == self.network_3['id']:
            port = self.network_3_port_1
        elif network_id == self.network_4['id']:
            port = self.network_4_port_1

        # this copy is here to avoid modifying class variables like
        # network_2_port_1 below at the update call
        port = copy.deepcopy(port)
        port.update(body['port'])

        # the tenant ID is normally extracted from credentials in the request
        # and is not present in the body
        if 'tenant_id' not in port:
            port['tenant_id'] = nova_fixtures.NeutronFixture.tenant_id

        # similarly, these attributes are set by neutron itself
        port['admin_state_up'] = True

        self._ports[port['id']] = port
        # this copy is here as nova sometimes modifies the returned port
        # locally and we want to avoid that nova modifies the fixture internals
        return {'port': copy.deepcopy(port)}



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_utils.fixture import uuidsentinel as uuids
from oslo_utils import timeutils

from nova import context as nova_context
from nova.network import model as network_model
from nova.notifications import base as notification_base
from nova.notifications.objects import instance as instance_notification
from nova import objects
from nova import test
from nova.tests.unit import fake_instance


class TestInstanceNotification(test.NoDBTestCase):

    def setUp(self):
        super(TestInstanceNotification, self).setUp()
        self.test_keys = ['memory_mb', 'vcpus', 'root_gb', 'ephemeral_gb',
                          'swap']
        self.flavor_values = {k: 123 for k in self.test_keys}
        instance_values = {k: 456 for k in self.test_keys}
        flavor = objects.Flavor(flavorid='test-flavor', name='test-flavor',
                                disabled=False, projects=[], is_public=True,
                                extra_specs={}, **self.flavor_values)
        info_cache = objects.InstanceInfoCache(
            network_info=network_model.NetworkInfo())
        self.instance = objects.Instance(
            flavor=flavor,
            info_cache=info_cache,
            metadata={},
            uuid=uuids.instance1,
            locked=False,
            auto_disk_config=False,
            system_metadata={},
            **instance_values)
        self.payload = {
            'bandwidth': {},
            'audit_period_ending': timeutils.utcnow(),
            'audit_period_beginning': timeutils.utcnow(),
        }

    @mock.patch('nova.notifications.objects.instance.'
                'InstanceUpdateNotification._emit')
    def test_send_version_instance_update_uses_flavor(self, mock_emit):
        # instance.update notification needs some tags value to avoid lazy-load
        self.instance.tags = objects.TagList()
        # Make sure that the notification payload chooses the values in
        # instance.flavor.$value instead of instance.$value
        mock_context = mock.MagicMock()
        mock_context.project_id = 'fake_project_id'
        mock_context.user_id = 'fake_user_id'
        mock_context.request_id = 'fake_req_id'
        notification_base._send_versioned_instance_update(
            mock_context,
            self.instance,
            self.payload,
            'host',
            'compute')
        payload = mock_emit.call_args_list[0][1]['payload']['nova_object.data']
        flavor_payload = payload['flavor']['nova_object.data']
        data = {k: flavor_payload[k] for k in self.test_keys}
        self.assertEqual(self.flavor_values, data)

    @mock.patch('nova.rpc.NOTIFIER')
    @mock.patch('nova.notifications.objects.instance.'
                'InstanceUpdatePayload.__init__', return_value=None)
    @mock.patch('nova.notifications.objects.instance.'
                'InstanceUpdateNotification.__init__', return_value=None)
    def test_send_versioned_instance_notification_is_not_called_disabled(
            self, mock_notification, mock_payload, mock_notifier):
        mock_notifier.is_enabled.return_value = False

        notification_base._send_versioned_instance_update(
            mock.MagicMock(),
            self.instance,
            self.payload,
            'host',
            'compute')

        self.assertFalse(mock_payload.called)
        self.assertFalse(mock_notification.called)

    @mock.patch('nova.notifications.objects.instance.'
                'InstanceUpdatePayload.__init__', return_value=None)
    @mock.patch('nova.notifications.objects.instance.'
                'InstanceUpdateNotification.__init__', return_value=None)
    def test_send_versioned_instance_notification_is_not_called_unversioned(
            self, mock_notification, mock_payload):
        self.flags(notification_format='unversioned', group='notifications')

        notification_base._send_versioned_instance_update(
            mock.MagicMock(),
            self.instance,
            self.payload,
            'host',
            'compute')

        self.assertFalse(mock_payload.called)
        self.assertFalse(mock_notification.called)

    def test_instance_payload_request_id_periodic_task(self):
        """Tests that creating an InstancePayload from the type of request
        context used during a periodic task will not populate the
        payload request_id field since it is not an end user request.
        """
        ctxt = nova_context.get_admin_context()
        instance = fake_instance.fake_instance_obj(ctxt)
        # Set some other fields otherwise populate_schema tries to hit the DB.
        instance.metadata = {}
        instance.system_metadata = {}
        instance.info_cache = objects.InstanceInfoCache(
            network_info=network_model.NetworkInfo([]))
        payload = instance_notification.InstancePayload(ctxt, instance)
        self.assertIsNone(payload.request_id)


class TestBlockDevicePayload(test.NoDBTestCase):
    @mock.patch('nova.objects.instance.Instance.get_bdms')
    def test_payload_contains_volume_bdms_if_requested(self, mock_get_bdms):
        self.flags(bdms_in_notifications='True', group='notifications')
        context = mock.Mock()
        instance = objects.Instance(uuid=uuids.instance_uuid)
        image_bdm = objects.BlockDeviceMapping(
            **{'context': context, 'source_type': 'image',
               'destination_type': 'local',
               'image_id': uuids.image_id,
               'volume_id': None,
               'device_name': '/dev/vda',
               'instance_uuid': instance.uuid})

        volume_bdm = objects.BlockDeviceMapping(
            **{'context': context, 'source_type': 'volume',
               'destination_type': 'volume',
               'volume_id': uuids.volume_id,
               'device_name': '/dev/vdb',
               'instance_uuid': instance.uuid,
               'boot_index': 0,
               'delete_on_termination': True,
               'tag': 'my-tag'})

        mock_get_bdms.return_value = [image_bdm, volume_bdm]

        bdms = instance_notification.BlockDevicePayload.from_instance(
            instance)

        self.assertEqual(1, len(bdms))
        bdm = bdms[0]
        self.assertIsInstance(bdm, instance_notification.BlockDevicePayload)
        self.assertEqual('/dev/vdb', bdm.device_name)
        self.assertEqual(0, bdm.boot_index)
        self.assertTrue(bdm.delete_on_termination)
        self.assertEqual('my-tag', bdm.tag)
        self.assertEqual(uuids.volume_id, bdm.volume_id)

    @mock.patch('nova.objects.instance.Instance.get_bdms',
                return_value=mock.NonCallableMock())
    def test_bdms_are_skipped_by_default(self, mock_get_bdms):
        instance = objects.Instance(uuid=uuids.instance_uuid)
        bmds = instance_notification.BlockDevicePayload.from_instance(
            instance)
        self.assertIsNone(bmds)



# Copyright 2018 Michael Still and Aptira
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Helpers for qemu tasks.
"""

import contextlib
import os
import tempfile
import typing as ty

from oslo_concurrency import processutils
from oslo_log import log as logging
from oslo_utils import units

from nova import exception
from nova.i18n import _
import nova.privsep.utils

LOG = logging.getLogger(__name__)

QEMU_IMG_LIMITS = processutils.ProcessLimits(
    cpu_time=30,
    address_space=1 * units.Gi)


class EncryptionOptions(ty.TypedDict):
    secret: str
    format: str


@nova.privsep.sys_admin_pctxt.entrypoint
def convert_image(source, dest, in_format, out_format, instances_path,
                  compress, src_encryption=None, dest_encryption=None):
    unprivileged_convert_image(source, dest, in_format, out_format,
                               instances_path, compress,
                               src_encryption=src_encryption,
                               dest_encryption=dest_encryption)


# NOTE(mikal): this method is deliberately not wrapped in a privsep entrypoint
def unprivileged_convert_image(
    source: str,
    dest: str,
    in_format: ty.Optional[str],
    out_format: str,
    instances_path: str,
    compress: bool,
    src_encryption: ty.Optional[EncryptionOptions] = None,
    dest_encryption: ty.Optional[EncryptionOptions] = None,
) -> None:
    """Disk image conversion with qemu-img

    :param source: Location of the disk image to convert
    :param dest: Desired location of the converted disk image
    :param in_format: Disk image format of the source image
    :param out_format: Desired disk image format of the converted disk image
    :param instances_path: Location where instances are stored on disk
    :param compress: Whether to compress the converted disk image
    :param src_encryption: (Optional) Dict detailing various encryption
        attributes for the source image such as the format and passphrase.
    :param dest_encryption: (Optional) Dict detailing various encryption
        attributes for the destination image such as the format and passphrase.

    The in_format and out_format represent disk image file formats in QEMU,
    which are:

        * qcow2, which can be encrypted or not encrypted depending on options
        * raw, which is unencrypted
        * luks, which is encrypted raw

    See https://www.qemu.org/docs/master/system/qemu-block-drivers.html
    """
    # NOTE(mdbooth, kchamart): `qemu-img convert` defaults to
    # 'cache=writeback' for the source image, and 'cache=unsafe' for the
    # target, which means that data is not synced to disk at completion.
    # We explicitly use 'cache=none' here, for the target image, to (1)
    # ensure that we don't interfere with other applications using the
    # host's I/O cache, and (2) ensure that the data is on persistent
    # storage when the command exits. Without (2), a host crash may
    # leave a corrupt image in the image cache, which Nova cannot
    # recover automatically.

    # NOTE(zigo, kchamart): We cannot use `qemu-img convert -t none` if
    # the 'instance_dir' is mounted on a filesystem that doesn't support
    # O_DIRECT, which is the case, for example, with 'tmpfs'. This
    # simply crashes `openstack server create` in environments like live
    # distributions. In such cases, the best choice is 'writeback',
    # which (a) makes the conversion multiple times faster; and (b) is
    # as safe as it can be, because at the end of the conversion it,
    # just like 'writethrough', calls fsync(2)|fdatasync(2), which
    # ensures to safely write the data to the physical disk.

    # NOTE(mikal): there is an assumption here that the source and destination
    # are in the instances_path. Is that worth enforcing?
    if nova.privsep.utils.supports_direct_io(instances_path):
        cache_mode = 'none'
    else:
        cache_mode = 'writeback'
    cmd = ['qemu-img', 'convert', '-t', cache_mode, '-O', out_format]

    # qemu-img: --image-opts and --format are mutually exclusive
    # If the source is encrypted, we will need to pass encryption related
    # options using --image-opts.
    driver_str = ''
    if in_format is not None:
        if not src_encryption:
            cmd += ['-f', in_format]
        else:
            driver_str = f'driver={in_format},'

    if compress:
        cmd += ['-c']

    src_secret_file = None
    dest_secret_file = None
    encryption_opts: ty.List[str] = []
    with contextlib.ExitStack() as stack:
        if src_encryption:
            src_secret_file = stack.enter_context(
                tempfile.NamedTemporaryFile(mode='tr+', encoding='utf-8'))

            # Write out the passphrase secret to a temp file
            src_secret_file.write(src_encryption['secret'])

            # Ensure the secret is written to disk, we can't .close() here as
            # that removes the file when using NamedTemporaryFile
            src_secret_file.flush()

            # When --image-opts is used, the source filename must be passed as
            # part of the option string instead of as a positional arg.
            #
            # The basic options include the secret and encryption format
            # Option names depend on the QEMU disk image file format:
            # https://www.qemu.org/docs/master/system/qemu-block-drivers.html#disk-image-file-formats # noqa
            # For 'luks' it is 'key-secret' and format is implied
            # For 'qcow2' it is 'encrypt.key-secret' and 'encrypt.format'
            prefix = 'encrypt.' if in_format == 'qcow2' else ''
            encryption_opts = [
                '--object', f"secret,id=sec0,file={src_secret_file.name}",
                '--image-opts',
                f"{driver_str}file.driver=file,file.filename={source},"
                f"{prefix}key-secret=sec0",
            ]

        if dest_encryption:
            dest_secret_file = stack.enter_context(
                tempfile.NamedTemporaryFile(mode='tr+', encoding='utf-8'))

            # Write out the passphrase secret to a temp file
            dest_secret_file.write(dest_encryption['secret'])

            # Ensure the secret is written to disk, we can't .close()
            # here as that removes the file when using
            # NamedTemporaryFile
            dest_secret_file.flush()

            prefix = 'encrypt.' if out_format == 'qcow2' else ''
            encryption_opts += [
                '--object', f"secret,id=sec1,file={dest_secret_file.name}",
                '-o', f'{prefix}key-secret=sec1',
            ]
            if prefix:
                # The encryption format is only relevant for the 'qcow2' disk
                # format. Otherwise, the disk format is 'luks' and the
                # encryption format is implied and not accepted as an option in
                # that case.
                encryption_opts += [
                    '-o', f"{prefix}format={dest_encryption['format']}"
                ]
            # Supported luks options:
            #  cipher-alg=<str>       - Name of cipher algorithm and
            #                           key length
            #  cipher-mode=<str>      - Name of encryption cipher mode
            #  hash-alg=<str>         - Name of hash algorithm to use
            #                           for PBKDF
            #  iter-time=<num>        - Time to spend in PBKDF in
            #                           milliseconds
            #  ivgen-alg=<str>        - Name of IV generator algorithm
            #  ivgen-hash-alg=<str>   - Name of IV generator hash
            #                           algorithm
            #
            # NOTE(melwitt): Sensible defaults (that match the qemu
            # defaults) are hardcoded at this time for simplicity and
            # consistency when instances are migrated. Configuration of
            # luks options could be added in a future release.
            encryption_options = {
                'cipher-alg': 'aes-256',
                'cipher-mode': 'xts',
                'hash-alg': 'sha256',
                'iter-time': 2000,
                'ivgen-alg': 'plain64',
                'ivgen-hash-alg': 'sha256',
            }
            for option, value in encryption_options.items():
                encryption_opts += [
                    '-o', f'{prefix}{option}={value}',
                ]

        if src_encryption or dest_encryption:
            cmd += encryption_opts

        # If the source is not encrypted, it's passed as a positional argument.
        if not src_encryption:
            cmd += [source]

        processutils.execute(*cmd + [dest])


@nova.privsep.sys_admin_pctxt.entrypoint
def privileged_qemu_img_info(path, format=None):
    """Return an object containing the parsed output from qemu-img info

    This is a privileged call to qemu-img info using the sys_admin_pctxt
    entrypoint allowing host block devices etc to be accessed.
    """
    return unprivileged_qemu_img_info(path, format=format)


def unprivileged_qemu_img_info(path, format=None):
    """Return an object containing the parsed output from qemu-img info."""
    try:
        # The following check is about ploop images that reside within
        # directories and always have DiskDescriptor.xml file beside them
        if (os.path.isdir(path) and
            os.path.exists(os.path.join(path, "DiskDescriptor.xml"))):
            path = os.path.join(path, "root.hds")

        cmd = [
            'env', 'LC_ALL=C', 'LANG=C', 'qemu-img', 'info', path,
            '--force-share', '--output=json',
            ]
        if format is not None:
            cmd = cmd + ['-f', format]
        out, err = processutils.execute(*cmd, prlimit=QEMU_IMG_LIMITS)
    except processutils.ProcessExecutionError as exp:
        if exp.exit_code == -9:
            # this means we hit prlimits, make the exception more specific
            msg = (_("qemu-img aborted by prlimits when inspecting "
                    "%(path)s : %(exp)s") % {'path': path, 'exp': exp})
        elif exp.exit_code == 1 and 'No such file or directory' in exp.stderr:
            # The os.path.exists check above can race so this is a simple
            # best effort at catching that type of failure and raising a more
            # specific error.
            raise exception.DiskNotFound(location=path)
        else:
            msg = (_("qemu-img failed to execute on %(path)s : %(exp)s") %
                   {'path': path, 'exp': exp})
        raise exception.InvalidDiskInfo(reason=msg)

    if not out:
        msg = (_("Failed to run qemu-img info on %(path)s : %(error)s") %
               {'path': path, 'error': err})
        raise exception.InvalidDiskInfo(reason=msg)
    return out



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class Console(object):
    def __init__(self, host, port, internal_access_path=None):
        self.host = host
        self.port = port
        self.internal_access_path = internal_access_path

    def get_connection_info(self, token, access_url):
        """Returns an unreferenced dict with connection information."""

        ret = dict(self.__dict__)
        ret['token'] = token
        ret['access_url'] = access_url
        return ret


class ConsoleVNC(Console):
    pass


class ConsoleSpice(Console):
    def __init__(self, host, port, tlsPort, internal_access_path=None):
        super(ConsoleSpice, self).__init__(host, port, internal_access_path)
        self.tlsPort = tlsPort


class ConsoleSerial(Console):
    pass


class ConsoleMKS(Console):
    pass



# Copyright 2015 Intel Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from nova.api import openstack
from nova.api.openstack import wsgi
from nova.tests.functional.api import client
from nova.tests.functional import integrated_helpers


class LegacyV2CompatibleTestBase(integrated_helpers._IntegratedTestBase):
    api_major_version = 'v2'

    def setUp(self):
        super(LegacyV2CompatibleTestBase, self).setUp()
        self._check_api_endpoint()

    def _check_api_endpoint(self):
        app = self.api_fixture.app().get((None, '/v2'))

        while getattr(app, 'application', False):
            if isinstance(
                app.application, openstack.LegacyV2CompatibleWrapper,
            ):
                break
            app = app.application
        else:
            raise Exception(
                'The LegacyV2CompatibleWrapper middleware is not configured.'
            )

    def test_request_with_microversion_headers(self):
        self.api.microversion = '2.100'
        response = self.api.api_post('os-keypairs',
            {"keypair": {"name": "test"}})
        self.assertNotIn(wsgi.API_VERSION_REQUEST_HEADER, response.headers)
        self.assertNotIn(wsgi.LEGACY_API_VERSION_REQUEST_HEADER,
                         response.headers)
        self.assertNotIn('Vary', response.headers)
        self.assertNotIn('type', response.body["keypair"])

    def test_request_without_additional_properties_check(self):
        self.api.microversion = '2.100'
        response = self.api.api_post('os-keypairs',
            {"keypair": {"name": "test", "foooooo": "barrrrrr"}})
        self.assertNotIn(wsgi.API_VERSION_REQUEST_HEADER, response.headers)
        self.assertNotIn(wsgi.LEGACY_API_VERSION_REQUEST_HEADER,
                         response.headers)
        self.assertNotIn('Vary', response.headers)
        self.assertNotIn('type', response.body["keypair"])

    def test_request_with_pattern_properties_check(self):
        server = self._build_server()
        post = {'server': server}
        created_server = self.api.post_server(post)
        self._wait_for_state_change(created_server, 'ACTIVE')
        response = self.api.post_server_metadata(created_server['id'],
                                                 {'a': 'b'})
        self.assertEqual(response, {'a': 'b'})

    def test_request_with_pattern_properties_with_avoid_metadata(self):
        server = self._build_server()
        post = {'server': server}
        created_server = self.api.post_server(post)
        exc = self.assertRaises(client.OpenStackApiException,
                                self.api.post_server_metadata,
                                created_server['id'],
                                {'a': 'b',
                                 'x' * 300: 'y',
                                 'h' * 300: 'i'})
        self.assertEqual(exc.response.status_code, 400)
        self.api.delete_server(created_server['id'])



# Copyright (c) 2022 SAP SE
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
# Copyright (c) 2012 VMware, Inc.
# Copyright (c) 2011 Citrix Systems, Inc.
# Copyright 2011 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


"""
Test suite for VMwareAPI Session
"""

from unittest import mock

from oslo_vmware import exceptions as vexec

from nova import test
from nova.tests.unit.virt.vmwareapi import fake as vmwareapi_fake
from nova.virt.vmwareapi import session


def _fake_create_session(inst):
    _session = vmwareapi_fake.DataObject()
    _session.key = 'fake_key'
    _session.userName = 'fake_username'
    _session._pbm_wsdl_loc = None
    _session._pbm = None
    inst._session = _session


def _fake_fetch_moref_impl(inst, _):
    inst.moref = vmwareapi_fake.ManagedObjectReference(
        value=mock.sentinel.moref2)


class FakeStableMoRefProxy(session.StableMoRefProxy):
    def __init__(self, ref=None):
        super(FakeStableMoRefProxy, self).__init__(
            ref or vmwareapi_fake.ManagedObjectReference(
                value=mock.sentinel.moref))

    def fetch_moref(self, session):
        pass

    def __repr__(self):
        return "FakeStableMoRefProxy({!r})".format(self.moref)


class StableMoRefProxyTestCase(test.NoDBTestCase):
    def test_proxy(self):
        ref = FakeStableMoRefProxy()
        self.assertEqual(mock.sentinel.moref, ref.value)
        self.assertEqual("ManagedObject", ref._type)

    def test_proxy_classes(self):
        # Necessary for suds serialisation
        ref = FakeStableMoRefProxy()
        self.assertEqual("ManagedObjectReference", ref.__class__.__name__)


class VMwareSessionTestCase(test.NoDBTestCase):

    @mock.patch.object(session.VMwareAPISession, '_is_vim_object',
                       return_value=False)
    def test_call_method(self, mock_is_vim):
        with test.nested(
            mock.patch.object(session.VMwareAPISession,
                              '_create_session',
                              _fake_create_session),
            mock.patch.object(session.VMwareAPISession,
                              'invoke_api'),
        ) as (fake_create, fake_invoke):
            _session = session.VMwareAPISession()
            _session._vim = mock.Mock()
            module = mock.Mock()
            _session._call_method(module, 'fira')
            fake_invoke.assert_called_once_with(module, 'fira', _session._vim)

    @mock.patch.object(session.VMwareAPISession, '_is_vim_object',
                       return_value=True)
    def test_call_method_vim(self, mock_is_vim):
        with test.nested(
            mock.patch.object(session.VMwareAPISession,
                              '_create_session',
                              _fake_create_session),
            mock.patch.object(session.VMwareAPISession,
                              'invoke_api'),
        ) as (fake_create, fake_invoke):
            _session = session.VMwareAPISession()
            module = mock.Mock()
            _session._call_method(module, 'fira')
            fake_invoke.assert_called_once_with(module, 'fira')

    @mock.patch.object(session.VMwareAPISession, '_is_vim_object',
                       return_value=True)
    def test_call_method_no_recovery(self, mock_is_vim):
        with test.nested(
            mock.patch.object(session.VMwareAPISession, '_create_session',
                              _fake_create_session),
            mock.patch.object(session.VMwareAPISession, 'invoke_api'),
            mock.patch.object(FakeStableMoRefProxy, 'fetch_moref'),
        ) as (fake_create, fake_invoke, fake_fetch_moref):
            _session = session.VMwareAPISession()
            module = mock.Mock()
            ref = FakeStableMoRefProxy()

            _session._call_method(
                module, mock.sentinel.method_arg, ref, ref=ref)

            fake_invoke.assert_called_once_with(
                module, mock.sentinel.method_arg, ref, ref=ref)
            fake_fetch_moref.assert_not_called()

    @mock.patch.object(session.VMwareAPISession, '_is_vim_object',
                       return_value=True)
    def test_call_method_recovery_arg_failed(self, mock_is_vim):
        with test.nested(
            mock.patch.object(session.VMwareAPISession, '_create_session',
                              _fake_create_session),
            mock.patch.object(session.VMwareAPISession, 'invoke_api'),
            mock.patch.object(FakeStableMoRefProxy, 'fetch_moref'),
        ) as (fake_create, fake_invoke, fake_fetch_moref):
            _session = session.VMwareAPISession()
            module = mock.Mock()
            ref = FakeStableMoRefProxy()
            fake_invoke.side_effect = [vexec.ManagedObjectNotFoundException]

            self.assertRaises(vexec.ManagedObjectNotFoundException,
                _session._call_method, module, mock.sentinel.method_arg, ref)

            fake_invoke.assert_called_once_with(
                module, mock.sentinel.method_arg, ref)
            fake_fetch_moref.assert_not_called()

    @mock.patch.object(session.VMwareAPISession, '_is_vim_object',
                       return_value=True)
    def test_call_method_recovery_kwarg_failed(self, mock_is_vim):
        with test.nested(
            mock.patch.object(session.VMwareAPISession, '_create_session',
                              _fake_create_session),
            mock.patch.object(session.VMwareAPISession, 'invoke_api'),
            mock.patch.object(FakeStableMoRefProxy, 'fetch_moref'),
        ) as (fake_create, fake_invoke, fake_fetch_moref):
            _session = session.VMwareAPISession()
            module = mock.Mock()
            ref = FakeStableMoRefProxy()
            fake_invoke.side_effect = [vexec.ManagedObjectNotFoundException]

            self.assertRaises(vexec.ManagedObjectNotFoundException,
                              _session._call_method, module,
                              mock.sentinel.method_arg, ref=ref)

            fake_invoke.assert_called_once_with(
                module, mock.sentinel.method_arg, ref=ref)
            fake_fetch_moref.assert_not_called()

    @mock.patch.object(session.VMwareAPISession, '_is_vim_object',
                       return_value=True)
    def test_call_method_recovery_arg_success(self, mock_is_vim):
        with test.nested(
            mock.patch.object(session.VMwareAPISession, '_create_session',
                              _fake_create_session),
            mock.patch.object(session.VMwareAPISession, 'invoke_api'),
            mock.patch.object(FakeStableMoRefProxy,
                              'fetch_moref', _fake_fetch_moref_impl),
        ) as (fake_create, fake_invoke, fake_fetch_moref):
            _session = session.VMwareAPISession()
            module = mock.Mock()
            ref = FakeStableMoRefProxy()

            fake_invoke.side_effect = [vexec.ManagedObjectNotFoundException(
                details=dict(obj=mock.sentinel.moref),
            ), None]
            _session._call_method(module, mock.sentinel.method_arg, ref)
            fake_invoke.assert_called_with(
                module, mock.sentinel.method_arg, ref)

    @mock.patch.object(session.VMwareAPISession, '_is_vim_object',
                       return_value=True)
    def test_call_method_recovery_kwarg_success(self, mock_is_vim):
        with test.nested(
            mock.patch.object(session.VMwareAPISession, '_create_session',
                              _fake_create_session),
            mock.patch.object(session.VMwareAPISession, 'invoke_api'),
            mock.patch.object(FakeStableMoRefProxy,
                                  'fetch_moref', _fake_fetch_moref_impl),
        ) as (fake_create, fake_invoke, fake_fetch_moref):
            _session = session.VMwareAPISession()
            module = mock.Mock()
            ref = FakeStableMoRefProxy()

            fake_invoke.side_effect = [vexec.ManagedObjectNotFoundException(
                details=dict(obj=mock.sentinel.moref),
            ), None]
            _session._call_method(module, mock.sentinel.method_arg, ref=ref)
            fake_invoke.assert_called_with(
                module, mock.sentinel.method_arg, ref=ref)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools
from unittest import mock

import fixtures
from oslo_utils.fixture import uuidsentinel as uuids
from oslo_utils import timeutils

from nova.api.openstack.compute import migrate_server
from nova.api.openstack.compute import servers
from nova.compute import api as compute
from nova.compute import vm_states
import nova.conf
from nova import exception
from nova.network import model
from nova.network import neutron
from nova import objects
from nova.objects import fields
from nova.objects.instance_group import InstanceGroup
from nova.policies import base as base_policy
from nova.policies import extended_server_attributes as ea_policies
from nova.policies import servers as policies
from nova.tests.unit.api.openstack import fakes
from nova.tests.unit import fake_flavor
from nova.tests.unit import fake_instance
from nova.tests.unit.policies import base

CONF = nova.conf.CONF


class ServersPolicyTest(base.BasePolicyTest):
    """Test Servers APIs policies with all possible context.
    This class defines the set of context with different roles
    which are allowed and not allowed to pass the policy checks.
    With those set of context, it will call the API operation and
    verify the expected behaviour.
    """

    def setUp(self):
        super(ServersPolicyTest, self).setUp()
        self.controller = servers.ServersController()
        self.m_controller = migrate_server.MigrateServerController()
        self.rule_trusted_certs = policies.SERVERS % 'create:trusted_certs'
        self.rule_attach_network = policies.SERVERS % 'create:attach_network'
        self.rule_attach_volume = policies.SERVERS % 'create:attach_volume'
        self.rule_requested_destination = policies.REQUESTED_DESTINATION
        self.rule_forced_host = policies.SERVERS % 'create:forced_host'

        self.req = fakes.HTTPRequest.blank('')
        user_id = self.req.environ['nova.context'].user_id

        self.controller._view_builder._add_security_grps = mock.MagicMock()
        self.controller._view_builder._get_metadata = mock.MagicMock()
        self.controller._view_builder._get_addresses = mock.MagicMock()
        self.controller._view_builder._get_host_id = mock.MagicMock()
        self.controller._view_builder._get_fault = mock.MagicMock()

        self.instance = fake_instance.fake_instance_obj(
                self.project_member_context,
                id=1, uuid=uuids.fake_id, project_id=self.project_id,
                user_id=user_id, vm_state=vm_states.ACTIVE,
                system_metadata={}, expected_attrs=['system_metadata'])

        self.mock_flavor = self.useFixture(
            fixtures.MockPatch('nova.compute.flavors.get_flavor_by_flavor_id'
                )).mock
        self.mock_flavor.return_value = fake_flavor.fake_flavor_obj(
                self.req.environ['nova.context'], flavorid='1')

        self.mock_get = self.useFixture(
            fixtures.MockPatch('nova.api.openstack.common.get_instance')).mock
        self.mock_get.return_value = self.instance

        self.mock_get_instance = self.useFixture(fixtures.MockPatchObject(
            self.controller, '_get_instance')).mock
        self.mock_get_instance.return_value = self.instance

        self.mock_get_network_info = self.useFixture(
            fixtures.MockPatch('nova.objects.Instance.get_network_info')).mock
        self.mock_get_network_info.return_value = model.NetworkInfo()

        self.servers = [fakes.stub_instance_obj(
            1, vm_state=vm_states.ACTIVE, uuid=uuids.fake,
            project_id=self.project_id, user_id='user1'),
                        fakes.stub_instance_obj(
            2, vm_state=vm_states.ACTIVE, uuid=uuids.fake,
            project_id='proj2', user_id='user2')]
        fakes.stub_out_secgroup_api(
            self, security_groups=[{'name': 'default'}])
        self.mock_get_all = self.useFixture(fixtures.MockPatchObject(
            self.controller.compute_api, 'get_all')).mock
        self.body = {
            'server': {
                'name': 'server_test',
                'imageRef': uuids.fake_id,
                'flavorRef': uuids.fake_id,
            },
        }
        self.extended_attr = ['OS-EXT-SRV-ATTR:host',
            'OS-EXT-SRV-ATTR:hypervisor_hostname',
            'OS-EXT-SRV-ATTR:instance_name',
            'OS-EXT-SRV-ATTR:hostname',
            'OS-EXT-SRV-ATTR:kernel_id',
            'OS-EXT-SRV-ATTR:launch_index',
            'OS-EXT-SRV-ATTR:ramdisk_id',
            'OS-EXT-SRV-ATTR:reservation_id',
            'OS-EXT-SRV-ATTR:root_device_name',
            'OS-EXT-SRV-ATTR:user_data'
        ]

        # Users that can take action on *our* project resources
        self.project_action_authorized_contexts = set([
            self.legacy_admin_context, self.system_admin_context,
            self.project_admin_context, self.project_member_context,
            self.project_reader_context, self.project_foo_context,
        ])

        # Users that can read *our* project resources
        self.project_reader_authorized_contexts = (
            self.project_action_authorized_contexts)

        # Users that _see_ project-scoped resources that they own
        self.everyone_authorized_contexts = set(self.all_contexts)

        # Users that can _do_ things to project-scoped resources they own
        self.project_member_authorized_contexts = set(self.all_contexts)

        # Users able to do admin things on project resources
        self.project_admin_authorized_contexts = set([
            self.legacy_admin_context, self.system_admin_context,
            self.project_admin_context])

        # Admin (for APIs does not pass the project id as policy target
        # for example, create server, list detail server) able to get
        # all projects servers, create server on specific host etc.
        # This is admin on any project because policy does not check
        # the project id but they will be able to create server, get
        # servers(unless all-tenant policy is allowed) of their own
        # project only.
        self.all_projects_admin_authorized_contexts = set([
            self.legacy_admin_context, self.system_admin_context,
            self.project_admin_context])

        # Users able to do cross-cell migrations
        self.cross_cell_authorized_contexts = []

    def test_index_server_policy(self):

        def fake_get_all(context, search_opts=None,
                         limit=None, marker=None,
                         expected_attrs=None, sort_keys=None, sort_dirs=None,
                         cell_down_support=False, all_tenants=False):
            self.assertIsNotNone(search_opts)
            if 'project_id' in search_opts or 'user_id' in search_opts:
                return objects.InstanceList(objects=self.servers)
            else:
                raise

        self.mock_get_all.side_effect = fake_get_all

        rule_name = policies.SERVERS % 'index'
        self.common_policy_auth(
            self.everyone_authorized_contexts,
            rule_name,
            self.controller.index,
            self.req)

    def test_index_all_project_server_policy(self):
        # 'index' policy is checked before 'index:get_all_tenants' so
        # we have to allow it for everyone otherwise it will
        # fail for unauthorized contexts here.
        rule = policies.SERVERS % 'index'
        self.policy.set_rules({rule: "@"}, overwrite=False)
        rule_name = policies.SERVERS % 'index:get_all_tenants'
        req = fakes.HTTPRequest.blank('/servers?all_tenants')

        def fake_get_all(context, search_opts=None,
                         limit=None, marker=None,
                         expected_attrs=None, sort_keys=None, sort_dirs=None,
                         cell_down_support=False, all_tenants=False):
            self.assertIsNotNone(search_opts)
            self.assertNotIn('project_id', search_opts)
            return objects.InstanceList(objects=self.servers)

        self.mock_get_all.side_effect = fake_get_all

        if not CONF.oslo_policy.enforce_scope:
            check_rule = rule_name
        else:
            check_rule = functools.partial(
                base.rule_if_system, rule, rule_name)

        self.common_policy_auth(self.all_projects_admin_authorized_contexts,
                                check_rule,
                                self.controller.index,
                                req)

    @mock.patch('nova.compute.api.API.get_all')
    def test_detail_list_server_policy(self, mock_get):

        def fake_get_all(context, search_opts=None,
                         limit=None, marker=None,
                         expected_attrs=None, sort_keys=None, sort_dirs=None,
                         cell_down_support=False, all_tenants=False):
            self.assertIsNotNone(search_opts)
            if 'project_id' in search_opts or 'user_id' in search_opts:
                return objects.InstanceList(objects=self.servers)
            else:
                raise

        self.mock_get_all.side_effect = fake_get_all

        rule_name = policies.SERVERS % 'detail'
        self.common_policy_auth(
            self.everyone_authorized_contexts,
            rule_name,
            self.controller.detail,
            self.req)

    def test_detail_list_all_project_server_policy(self):
        # 'detail' policy is checked before 'detail:get_all_tenants' so
        # we have to allow it for everyone otherwise it will
        # fail for unauthorized contexts here.
        rule = policies.SERVERS % 'detail'
        self.policy.set_rules({rule: "@"}, overwrite=False)
        rule_name = policies.SERVERS % 'detail:get_all_tenants'
        req = fakes.HTTPRequest.blank('/servers?all_tenants')

        def fake_get_all(context, search_opts=None,
                         limit=None, marker=None,
                         expected_attrs=None, sort_keys=None, sort_dirs=None,
                         cell_down_support=False, all_tenants=False):
            self.assertIsNotNone(search_opts)
            self.assertNotIn('project_id', search_opts)
            return objects.InstanceList(objects=self.servers)

        self.mock_get_all.side_effect = fake_get_all

        if not CONF.oslo_policy.enforce_scope:
            check_rule = rule_name
        else:
            check_rule = functools.partial(
                base.rule_if_system, rule, rule_name)

        self.common_policy_auth(self.all_projects_admin_authorized_contexts,
                                check_rule,
                                self.controller.detail,
                                req)

    def test_index_server_allow_all_filters_policy(self):
        # 'index' policy is checked before 'allow_all_filters' so
        # we have to allow it for everyone otherwise it will
        # fail for unauthorized contexts here.
        rule = policies.SERVERS % 'index'
        self.policy.set_rules({rule: "@"}, overwrite=False)

        def fake_get_all(context, search_opts=None,
                         limit=None, marker=None,
                         expected_attrs=None, sort_keys=None, sort_dirs=None,
                         cell_down_support=False, all_tenants=False):
            self.assertIsNotNone(search_opts)
            if context not in self.all_projects_admin_authorized_contexts:
                self.assertNotIn('host', search_opts)
            if context in self.all_projects_admin_authorized_contexts:
                self.assertIn('host', search_opts)
            return objects.InstanceList(objects=self.servers)

        self.mock_get_all.side_effect = fake_get_all

        req = fakes.HTTPRequest.blank('/servers?host=1')
        rule_name = policies.SERVERS % 'allow_all_filters'
        self.common_policy_auth(
            self.all_projects_admin_authorized_contexts,
            rule_name,
            self.controller.index,
            req, fatal=False)

    def test_detail_server_allow_all_filters_policy(self):
        # 'detail' policy is checked before 'allow_all_filters' so
        # we have to allow it for everyone otherwise it will
        # fail for unauthorized contexts here.
        rule = policies.SERVERS % 'detail'
        self.policy.set_rules({rule: "@"}, overwrite=False)

        def fake_get_all(context, search_opts=None,
                         limit=None, marker=None,
                         expected_attrs=None, sort_keys=None, sort_dirs=None,
                         cell_down_support=False, all_tenants=False):
            self.assertIsNotNone(search_opts)
            if context not in self.all_projects_admin_authorized_contexts:
                self.assertNotIn('host', search_opts)
            if context in self.all_projects_admin_authorized_contexts:
                self.assertIn('host', search_opts)
            return objects.InstanceList(objects=self.servers)
        self.mock_get_all.side_effect = fake_get_all

        req = fakes.HTTPRequest.blank('/servers?host=1')
        rule_name = policies.SERVERS % 'allow_all_filters'
        self.common_policy_auth(
            self.all_projects_admin_authorized_contexts,
            rule_name,
            self.controller.detail,
            req, fatal=False)

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    def test_show_server_policy(self, mock_bdm):
        rule_name = policies.SERVERS % 'show'
        # Show includes readers
        self.common_policy_auth(
            self.project_reader_authorized_contexts,
            rule_name,
            self.controller.show,
            self.req, self.instance.uuid)

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    @mock.patch('nova.compute.api.API.get_instance_host_status')
    def test_server_show_with_extra_specs_policy(self, mock_get, mock_block):
        rule = policies.SERVERS % 'show'
        # server 'show' policy is checked before flavor extra specs
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts.
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.47')
        rule_name = policies.SERVERS % 'show:flavor-extra-specs'
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.project_reader_authorized_contexts,
            rule_name, self.controller.show, req,
            self.instance.uuid, fatal=False)
        for resp in authorize_res:
            self.assertIn('extra_specs', resp['server']['flavor'])
        for resp in unauthorize_res:
            self.assertNotIn('extra_specs', resp['server']['flavor'])

    @mock.patch('nova.compute.api.API.get_all')
    def test_server_detail_with_extra_specs_policy(self, mock_get):

        def fake_get_all(context, search_opts=None,
                         limit=None, marker=None,
                         expected_attrs=None, sort_keys=None, sort_dirs=None,
                         cell_down_support=False, all_tenants=False):
            self.assertIsNotNone(search_opts)
            if 'project_id' in search_opts or 'user_id' in search_opts:
                return objects.InstanceList(objects=self.servers)
            else:
                raise

        self.mock_get_all.side_effect = fake_get_all
        rule = policies.SERVERS % 'detail'
        # server 'detail' policy is checked before flavor extra specs
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts.
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.47')
        rule_name = policies.SERVERS % 'show:flavor-extra-specs'
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.everyone_authorized_contexts,
            rule_name, self.controller.detail, req,
            fatal=False)
        for resp in authorize_res:
            self.assertIn('extra_specs', resp['servers'][0]['flavor'])
        for resp in unauthorize_res:
            self.assertNotIn('extra_specs', resp['servers'][0]['flavor'])

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    @mock.patch('nova.compute.api.API.get_instance_host_status')
    @mock.patch('nova.compute.api.API.rebuild')
    def test_server_rebuild_with_extra_specs_policy(self, mock_rebuild,
        mock_get, mock_bdm):
        rule = policies.SERVERS % 'rebuild'
        # server 'rebuild' policy is checked before flavor extra specs
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts.
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.47')
        rule_name = policies.SERVERS % 'show:flavor-extra-specs'
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.project_reader_authorized_contexts,
            rule_name, self.controller._action_rebuild,
            req, self.instance.uuid,
            body={'rebuild': {"imageRef": uuids.fake_id}},
            fatal=False)
        for resp in authorize_res:
            self.assertIn('extra_specs', resp.obj['server']['flavor'])
        for resp in unauthorize_res:
            self.assertNotIn('extra_specs', resp.obj['server']['flavor'])

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    @mock.patch.object(InstanceGroup, 'get_by_instance_uuid')
    @mock.patch('nova.compute.api.API.update_instance')
    def test_server_update_with_extra_specs_policy(self,
            mock_update, mock_group, mock_bdm):
        mock_update.return_value = self.instance
        rule = policies.SERVERS % 'update'
        # server 'update' policy is checked before flavor extra specs
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts.
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.47')
        rule_name = policies.SERVERS % 'show:flavor-extra-specs'
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.project_reader_authorized_contexts,
            rule_name, self.controller.update,
            req, self.instance.uuid,
            body={'server': {'name': 'test'}},
            fatal=False)
        for resp in authorize_res:
            self.assertIn('extra_specs', resp['server']['flavor'])
        for resp in unauthorize_res:
            self.assertNotIn('extra_specs', resp['server']['flavor'])

    @mock.patch('nova.compute.api.API.create')
    def test_create_server_policy(self, mock_create):
        mock_create.return_value = ([self.instance], '')
        rule_name = policies.SERVERS % 'create'
        self.common_policy_auth(self.project_member_authorized_contexts,
                                rule_name,
                                self.controller.create,
                                self.req, body=self.body)

    @mock.patch('nova.compute.api.API.create')
    @mock.patch('nova.compute.api.API.parse_availability_zone')
    @mock.patch.object(
        servers.ServersController, '_validate_host_availability_zone',
        new=mock.Mock(return_value=None))
    def test_create_forced_host_server_policy(self, mock_az, mock_create):
        # 'create' policy is checked before 'create:forced_host' so
        # we have to allow it for everyone otherwise it will
        # fail for unauthorized contexts here.
        rule = policies.SERVERS % 'create'
        self.policy.set_rules({rule: "@"}, overwrite=False)
        mock_create.return_value = ([self.instance], '')
        mock_az.return_value = ('test', 'host', None)
        self.common_policy_auth(self.all_projects_admin_authorized_contexts,
                                self.rule_forced_host,
                                self.controller.create,
                                self.req, body=self.body)

    @mock.patch('nova.compute.api.API.create')
    def test_create_attach_volume_server_policy(self, mock_create):
        # 'create' policy is checked before 'create:attach_volume' so
        # we have to allow it for everyone otherwise it will
        # fail for unauthorized contexts here.
        rule = policies.SERVERS % 'create'
        self.policy.set_rules({rule: "@"}, overwrite=False)
        mock_create.return_value = ([self.instance], '')
        body = {
            'server': {
                'name': 'server_test',
                'imageRef': uuids.fake_id,
                'flavorRef': uuids.fake_id,
                'block_device_mapping': [{'device_name': 'foo'}],
            },
        }
        self.common_policy_auth(self.project_member_authorized_contexts,
                                self.rule_attach_volume,
                                self.controller.create,
                                self.req, body=body)

    @mock.patch('nova.compute.api.API.create')
    def test_create_attach_network_server_policy(self, mock_create):
        # 'create' policy is checked before 'create:attach_network' so
        # we have to allow it for everyone otherwise it will
        # fail for unauthorized contexts here.
        rule = policies.SERVERS % 'create'
        self.policy.set_rules({rule: "@"}, overwrite=False)
        mock_create.return_value = ([self.instance], '')
        body = {
            'server': {
                'name': 'server_test',
                'imageRef': uuids.fake_id,
                'flavorRef': uuids.fake_id,
                'networks': [{
                    'uuid': uuids.fake_id
                }],
            },
        }
        self.common_policy_auth(self.project_member_authorized_contexts,
                                self.rule_attach_network,
                                self.controller.create,
                                self.req, body=body)

    @mock.patch('nova.compute.api.API.create')
    def test_create_trusted_certs_server_policy(self, mock_create):
        # 'create' policy is checked before 'create:trusted_certs' so
        # we have to allow it for everyone otherwise it will
        # fail for unauthorized contexts here.
        rule = policies.SERVERS % 'create'
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.63')
        mock_create.return_value = ([self.instance], '')
        body = {
            'server': {
                'name': 'server_test',
                'imageRef': uuids.fake_id,
                'flavorRef': uuids.fake_id,
                'trusted_image_certificates': [uuids.fake_id],
                'networks': [{
                    'uuid': uuids.fake_id
                }],

            },
        }
        self.common_policy_auth(self.project_member_authorized_contexts,
                                self.rule_trusted_certs,
                                self.controller.create,
                                req, body=body)

    @mock.patch('nova.compute.api.API.delete')
    def test_delete_server_policy(self, mock_delete):
        rule_name = policies.SERVERS % 'delete'
        self.common_policy_auth(self.project_action_authorized_contexts,
                                rule_name,
                                self.controller.delete,
                                self.req, self.instance.uuid)

    def test_delete_server_policy_failed_with_other_user(self):
        # Change the user_id in request context.
        req = fakes.HTTPRequest.blank('')
        req.environ['nova.context'].user_id = 'other-user'
        rule_name = policies.SERVERS % 'delete'
        self.policy.set_rules({rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        exc = self.assertRaises(
            exception.PolicyNotAuthorized, self.controller.delete,
            req, self.instance.uuid)
        self.assertEqual(
            "Policy doesn't allow %s to be performed." % rule_name,
            exc.format_message())

    @mock.patch('nova.compute.api.API.delete')
    def test_delete_server_overridden_policy_pass_with_same_user(
        self, mock_delete):
        rule_name = policies.SERVERS % 'delete'
        self.policy.set_rules({rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        self.controller.delete(self.req,
                               self.instance.uuid)

    @mock.patch('nova.compute.api.API.update_instance')
    def test_update_server_policy(self, mock_update):
        rule_name = policies.SERVERS % 'update'
        body = {'server': {'name': 'test'}}

        self.common_policy_auth(self.project_action_authorized_contexts,
                                rule_name,
                                self.controller.update,
                                self.req, self.instance.uuid, body=body)

    def test_update_server_policy_failed_with_other_user(self):
        # Change the user_id in request context.
        req = fakes.HTTPRequest.blank('')
        req.environ['nova.context'].user_id = 'other-user'
        rule_name = policies.SERVERS % 'update'
        body = {'server': {'name': 'test'}}
        self.policy.set_rules({rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        exc = self.assertRaises(
            exception.PolicyNotAuthorized, self.controller.update,
            req, self.instance.uuid, body=body)
        self.assertEqual(
            "Policy doesn't allow %s to be performed." % rule_name,
            exc.format_message())

    @mock.patch('nova.compute.api.API.update_instance')
    def test_update_server_overridden_policy_pass_with_same_user(
        self, mock_update):
        rule_name = policies.SERVERS % 'update'
        self.policy.set_rules({rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        body = {'server': {'name': 'test'}}
        self.controller.update(self.req,
                               self.instance.uuid, body=body)

    @mock.patch('nova.compute.api.API.confirm_resize')
    def test_confirm_resize_server_policy(self, mock_confirm_resize):
        rule_name = policies.SERVERS % 'confirm_resize'

        self.common_policy_auth(self.project_action_authorized_contexts,
                                rule_name,
                                self.controller._action_confirm_resize,
                                self.req, self.instance.uuid,
                                body={'confirmResize': 'null'})

    @mock.patch('nova.compute.api.API.revert_resize')
    def test_revert_resize_server_policy(self, mock_revert_resize):
        rule_name = policies.SERVERS % 'revert_resize'

        self.common_policy_auth(self.project_action_authorized_contexts,
                                rule_name,
                                self.controller._action_revert_resize,
                                self.req, self.instance.uuid,
                                body={'revertResize': 'null'})

    @mock.patch('nova.compute.api.API.reboot')
    def test_reboot_server_policy(self, mock_reboot):
        rule_name = policies.SERVERS % 'reboot'

        self.common_policy_auth(self.project_action_authorized_contexts,
                                rule_name,
                                self.controller._action_reboot,
                                self.req, self.instance.uuid,
                                body={'reboot': {'type': 'soft'}})

    @mock.patch('nova.compute.api.API.resize')
    def test_resize_server_policy(self, mock_resize):
        rule_name = policies.SERVERS % 'resize'
        self.common_policy_auth(self.project_action_authorized_contexts,
                                rule_name,
                                self.controller._action_resize,
                                self.req, self.instance.uuid,
                                body={'resize': {'flavorRef': 'f1'}})

    def test_resize_server_policy_failed_with_other_user(self):
        # Change the user_id in request context.
        req = fakes.HTTPRequest.blank('')
        req.environ['nova.context'].user_id = 'other-user'
        rule_name = policies.SERVERS % 'resize'
        body = {'resize': {'flavorRef': 'f1'}}
        self.policy.set_rules({rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        exc = self.assertRaises(
            exception.PolicyNotAuthorized, self.controller._action_resize,
            req, self.instance.uuid, body=body)
        self.assertEqual(
            "Policy doesn't allow %s to be performed." % rule_name,
            exc.format_message())

    @mock.patch('nova.compute.api.API.resize')
    def test_resize_server_overridden_policy_pass_with_same_user(
            self, mock_resize):
        rule_name = policies.SERVERS % 'resize'
        self.policy.set_rules({rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        body = {'resize': {'flavorRef': 'f1'}}
        self.controller._action_resize(self.req,
                                       self.instance.uuid, body=body)

    @mock.patch('nova.compute.api.API.start')
    def test_start_server_policy(self, mock_start):
        rule_name = policies.SERVERS % 'start'

        self.common_policy_auth(self.project_action_authorized_contexts,
                                rule_name,
                                self.controller._start_server,
                                self.req, self.instance.uuid,
                                body={'os-start': 'null'})

    @mock.patch('nova.compute.api.API.stop')
    def test_stop_server_policy(self, mock_stop):
        rule_name = policies.SERVERS % 'stop'

        self.common_policy_auth(self.project_action_authorized_contexts,
                                rule_name,
                                self.controller._stop_server,
                                self.req, self.instance.uuid,
                                body={'os-stop': 'null'})

    def test_stop_server_policy_failed_with_other_user(self):
        # Change the user_id in request context.
        req = fakes.HTTPRequest.blank('')
        req.environ['nova.context'].user_id = 'other-user'
        rule_name = policies.SERVERS % 'stop'
        body = {'os-stop': 'null'}
        self.policy.set_rules({rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        exc = self.assertRaises(
            exception.PolicyNotAuthorized, self.controller._stop_server,
            req, self.instance.uuid, body=body)
        self.assertEqual(
            "Policy doesn't allow %s to be performed." % rule_name,
            exc.format_message())

    @mock.patch('nova.compute.api.API.stop')
    def test_stop_server_overridden_policy_pass_with_same_user(
        self, mock_stop):
        rule_name = policies.SERVERS % 'stop'
        self.policy.set_rules({rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        body = {'os-stop': 'null'}
        self.controller._stop_server(self.req,
                                     self.instance.uuid, body=body)

    @mock.patch('nova.compute.api.API.rebuild')
    def test_rebuild_server_policy(self, mock_rebuild):
        rule_name = policies.SERVERS % 'rebuild'
        self.common_policy_auth(self.project_action_authorized_contexts,
                                rule_name,
                                self.controller._action_rebuild,
                                self.req, self.instance.uuid,
                                body={'rebuild': {"imageRef": uuids.fake_id}})

    def test_rebuild_server_policy_failed_with_other_user(self):
        # Change the user_id in request context.
        req = fakes.HTTPRequest.blank('')
        req.environ['nova.context'].user_id = 'other-user'
        rule_name = policies.SERVERS % 'rebuild'
        body = {'rebuild': {"imageRef": uuids.fake_id}}
        self.policy.set_rules({rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        exc = self.assertRaises(
            exception.PolicyNotAuthorized, self.controller._action_rebuild,
            req, self.instance.uuid, body=body)
        self.assertEqual(
            "Policy doesn't allow %s to be performed." % rule_name,
            exc.format_message())

    @mock.patch('nova.compute.api.API.rebuild')
    def test_rebuild_server_overridden_policy_pass_with_same_user(
        self, mock_rebuild):
        rule_name = policies.SERVERS % 'rebuild'
        self.policy.set_rules({rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        body = {'rebuild': {"imageRef": uuids.fake_id}}
        self.controller._action_rebuild(self.req,
                                        self.instance.uuid, body=body)

    @mock.patch('nova.compute.api.API.rebuild')
    def test_rebuild_trusted_certs_server_policy(self, mock_rebuild):
        # 'rebuild' policy is checked before 'rebuild:trusted_certs' so
        # we have to allow it for everyone otherwise it will
        # fail for unauthorized contexts here.
        rule = policies.SERVERS % 'rebuild'
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.63')
        rule_name = policies.SERVERS % 'rebuild:trusted_certs'
        body = {
            'rebuild': {
                'imageRef': uuids.fake_id,
                'trusted_image_certificates': [uuids.fake_id],
            },
        }

        if not CONF.oslo_policy.enforce_scope:
            check_rule = rule_name
        else:
            check_rule = functools.partial(
                base.rule_if_system, rule, rule_name)

        self.common_policy_auth(self.project_action_authorized_contexts,
                                check_rule,
                                self.controller._action_rebuild,
                                req, self.instance.uuid, body=body)

    def test_rebuild_trusted_certs_policy_failed_with_other_user(self):
        # Change the user_id in request context.
        req = fakes.HTTPRequest.blank('', version='2.63')
        req.environ['nova.context'].user_id = 'other-user'
        rule = policies.SERVERS % 'rebuild'
        rule_name = policies.SERVERS % 'rebuild:trusted_certs'
        body = {
            'rebuild': {
                'imageRef': uuids.fake_id,
                'trusted_image_certificates': [uuids.fake_id],
            },
        }
        self.policy.set_rules(
            {rule: "@",
             rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        exc = self.assertRaises(
            exception.PolicyNotAuthorized, self.controller._action_rebuild,
            req, self.instance.uuid, body=body)
        self.assertEqual(
            "Policy doesn't allow %s to be performed." % rule_name,
            exc.format_message())

    @mock.patch('nova.compute.api.API.rebuild')
    def test_rebuild_trusted_certs_overridden_policy_pass_with_same_user(
        self, mock_rebuild):
        req = fakes.HTTPRequest.blank('', version='2.63')
        rule = policies.SERVERS % 'rebuild'
        rule_name = policies.SERVERS % 'rebuild:trusted_certs'
        body = {
            'rebuild': {
                'imageRef': uuids.fake_id,
                'trusted_image_certificates': [uuids.fake_id],
            },
        }
        self.policy.set_rules(
            {rule: "@",
             rule_name: "user_id:%(user_id)s"}, overwrite=False)
        self.controller._action_rebuild(req,
                                        self.instance.uuid, body=body)

    @mock.patch('nova.objects.BlockDeviceMappingList.get_by_instance_uuid')
    @mock.patch('nova.image.glance.API.generate_image_url')
    @mock.patch('nova.compute.api.API.snapshot_volume_backed')
    def test_create_image_server_policy(self, mock_snapshot, mock_image,
        mock_bdm):
        rule_name = policies.SERVERS % 'create_image'
        self.common_policy_auth(self.project_action_authorized_contexts,
                                rule_name,
                                self.controller._action_create_image,
                                self.req, self.instance.uuid,
                                body={'createImage': {"name": 'test'}})

    @mock.patch('nova.objects.BlockDeviceMappingList.get_by_instance_uuid')
    @mock.patch('nova.image.glance.API.generate_image_url')
    @mock.patch('nova.compute.api.API.snapshot_volume_backed')
    def test_create_image_allow_volume_backed_server_policy(self,
        mock_snapshot, mock_image, mock_bdm):
        # 'create_image' policy is checked before
        # 'create_image:allow_volume_backed' so
        # we have to allow it for everyone otherwise it will
        # fail for unauthorized contexts here.
        rule = policies.SERVERS % 'create_image'
        self.policy.set_rules({rule: "@"}, overwrite=False)

        rule_name = policies.SERVERS % 'create_image:allow_volume_backed'
        if not CONF.oslo_policy.enforce_scope:
            check_rule = rule_name
        else:
            check_rule = functools.partial(
                base.rule_if_system, rule, rule_name)
        self.common_policy_auth(self.project_action_authorized_contexts,
                                check_rule,
                                self.controller._action_create_image,
                                self.req, self.instance.uuid,
                                body={'createImage': {"name": 'test'}})

    @mock.patch('nova.compute.api.API.trigger_crash_dump')
    def test_trigger_crash_dump_server_policy(self, mock_crash):
        rule_name = policies.SERVERS % 'trigger_crash_dump'
        req = fakes.HTTPRequest.blank('', version='2.17')
        self.common_policy_auth(self.project_action_authorized_contexts,
                                rule_name,
                                self.controller._action_trigger_crash_dump,
                                req, self.instance.uuid,
                                body={'trigger_crash_dump': None})

    def test_trigger_crash_dump_policy_failed_with_other_user(self):
        # Change the user_id in request context.
        req = fakes.HTTPRequest.blank('', version='2.17')
        req.environ['nova.context'].user_id = 'other-user'
        rule_name = policies.SERVERS % 'trigger_crash_dump'
        body = {'trigger_crash_dump': None}
        self.policy.set_rules({rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        exc = self.assertRaises(
            exception.PolicyNotAuthorized,
            self.controller._action_trigger_crash_dump,
            req, self.instance.uuid, body=body)
        self.assertEqual(
            "Policy doesn't allow %s to be performed." % rule_name,
            exc.format_message())

    @mock.patch('nova.compute.api.API.trigger_crash_dump')
    def test_trigger_crash_dump_overridden_policy_pass_with_same_user(
        self, mock_crash):
        req = fakes.HTTPRequest.blank('', version='2.17')
        rule_name = policies.SERVERS % 'trigger_crash_dump'
        self.policy.set_rules({rule_name: "user_id:%(user_id)s"},
            overwrite=False)
        body = {'trigger_crash_dump': None}
        self.controller._action_trigger_crash_dump(req,
            self.instance.uuid, body=body)

    def test_server_detail_with_extended_attr_policy(self):
        def fake_get_all(context, search_opts=None,
                         limit=None, marker=None,
                         expected_attrs=None, sort_keys=None, sort_dirs=None,
                         cell_down_support=False, all_tenants=False):
            return objects.InstanceList(objects=self.servers)
        self.mock_get_all.side_effect = fake_get_all

        rule = policies.SERVERS % 'detail'
        # server 'detail' policy is checked before extended attributes
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts.
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.3')
        rule_name = ea_policies.BASE_POLICY_NAME
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.all_projects_admin_authorized_contexts,
            rule_name, self.controller.detail, req,
            fatal=False)
        for attr in self.extended_attr:
            for resp in authorize_res:
                self.assertIn(attr, resp['servers'][0])
            for resp in unauthorize_res:
                self.assertNotIn(attr, resp['servers'][0])

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    @mock.patch('nova.compute.api.API.get_instance_host_status')
    def test_server_show_with_extended_attr_policy(self, mock_get, mock_block):
        rule = policies.SERVERS % 'show'
        # server 'show' policy is checked before extended attributes
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts.
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.3')
        rule_name = ea_policies.BASE_POLICY_NAME
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.project_admin_authorized_contexts,
            rule_name, self.controller.show, req, 'fake',
            fatal=False)
        for attr in self.extended_attr:
            for resp in authorize_res:
                self.assertIn(attr, resp['server'])
            for resp in unauthorize_res:
                self.assertNotIn(attr, resp['server'])

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    @mock.patch('nova.compute.api.API.get_instance_host_status')
    @mock.patch('nova.compute.api.API.rebuild')
    def test_server_rebuild_with_extended_attr_policy(self, mock_rebuild,
        mock_get, mock_bdm):
        rule = policies.SERVERS % 'rebuild'
        # server 'rebuild' policy is checked before extended attributes
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts.
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.75')
        rule_name = ea_policies.BASE_POLICY_NAME
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.project_admin_authorized_contexts,
            rule_name, self.controller._action_rebuild,
            req, self.instance.uuid,
            body={'rebuild': {"imageRef": uuids.fake_id}},
            fatal=False)
        for attr in self.extended_attr:
            # NOTE(gmann): user_data attribute is always present in
            # rebuild response since 2.47.
            if attr == 'OS-EXT-SRV-ATTR:user_data':
                continue
            for resp in authorize_res:
                self.assertIn(attr, resp.obj['server'])
            for resp in unauthorize_res:
                self.assertNotIn(attr, resp.obj['server'])

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    @mock.patch.object(InstanceGroup, 'get_by_instance_uuid')
    @mock.patch('nova.compute.api.API.update_instance')
    @mock.patch('nova.compute.api.API.get_instance_host_status')
    def test_server_update_with_extended_attr_policy(self,
        mock_status, mock_update, mock_group, mock_bdm):
        mock_update.return_value = self.instance
        mock_status.return_value = fields.HostStatus.UP
        rule = policies.SERVERS % 'update'
        # server 'update' policy is checked before extended attributes
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts.
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.75')
        rule_name = ea_policies.BASE_POLICY_NAME
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.project_admin_authorized_contexts,
            rule_name, self.controller.update,
            req, self.instance.uuid,
            body={'server': {'name': 'test'}},
            fatal=False)
        for attr in self.extended_attr:
            for resp in authorize_res:
                self.assertIn(attr, resp['server'])
            for resp in unauthorize_res:
                self.assertNotIn(attr, resp['server'])

    def test_server_detail_with_host_status_policy(self):
        def fake_get_all(context, search_opts=None,
                         limit=None, marker=None,
                         expected_attrs=None, sort_keys=None, sort_dirs=None,
                         cell_down_support=False, all_tenants=False):
            return objects.InstanceList(objects=self.servers)
        self.mock_get_all.side_effect = fake_get_all

        rule = policies.SERVERS % 'detail'
        # server 'detail' policy is checked before host_status
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts.
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.16')
        rule_name = policies.SERVERS % 'show:host_status'
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.all_projects_admin_authorized_contexts,
            rule_name, self.controller.detail, req,
            fatal=False)
        for resp in authorize_res:
            self.assertIn('host_status', resp['servers'][0])
        for resp in unauthorize_res:
            self.assertNotIn('host_status', resp['servers'][0])

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    @mock.patch('nova.compute.api.API.get_instance_host_status')
    def test_server_show_with_host_status_policy(self,
        mock_status, mock_block):
        rule = policies.SERVERS % 'show'
        # server 'show' policy is checked before host_status
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts.
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.16')
        rule_name = policies.SERVERS % 'show:host_status'
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.project_admin_authorized_contexts,
            rule_name, self.controller.show, req, 'fake',
            fatal=False)
        for resp in authorize_res:
            self.assertIn('host_status', resp['server'])
        for resp in unauthorize_res:
            self.assertNotIn('host_status', resp['server'])

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    @mock.patch('nova.compute.api.API.get_instance_host_status')
    @mock.patch('nova.compute.api.API.rebuild')
    def test_server_rebuild_with_host_status_policy(self, mock_rebuild,
        mock_status, mock_bdm):
        rule = policies.SERVERS % 'rebuild'
        # server 'rebuild' policy is checked before host_status
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts.
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.75')
        rule_name = policies.SERVERS % 'show:host_status'
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.project_admin_authorized_contexts,
            rule_name, self.controller._action_rebuild,
            req, self.instance.uuid,
            body={'rebuild': {"imageRef": uuids.fake_id}},
            fatal=False)
        for resp in authorize_res:
            self.assertIn('host_status', resp.obj['server'])
        for resp in unauthorize_res:
            self.assertNotIn('host_status', resp.obj['server'])

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    @mock.patch.object(InstanceGroup, 'get_by_instance_uuid')
    @mock.patch('nova.compute.api.API.update_instance')
    @mock.patch('nova.compute.api.API.get_instance_host_status')
    def test_server_update_with_host_status_policy(self,
        mock_status, mock_update, mock_group, mock_bdm):
        mock_update.return_value = self.instance
        mock_status.return_value = fields.HostStatus.UP
        rule = policies.SERVERS % 'update'
        # server 'update' policy is checked before host_status
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts.
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.75')
        rule_name = policies.SERVERS % 'show:host_status'
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.project_admin_authorized_contexts,
            rule_name, self.controller.update,
            req, self.instance.uuid,
            body={'server': {'name': 'test'}},
            fatal=False)
        for resp in authorize_res:
            self.assertIn('host_status', resp['server'])
        for resp in unauthorize_res:
            self.assertNotIn('host_status', resp['server'])

    @mock.patch('nova.compute.api.API.get_instances_host_statuses')
    def test_server_detail_with_unknown_host_status_policy(self, mock_status):
        def fake_get_all(context, search_opts=None,
                         limit=None, marker=None,
                         expected_attrs=None, sort_keys=None, sort_dirs=None,
                         cell_down_support=False, all_tenants=False):
            return objects.InstanceList(objects=self.servers)
        self.mock_get_all.side_effect = fake_get_all
        host_statuses = {}
        for server in self.servers:
            host_statuses.update({server.uuid: fields.HostStatus.UNKNOWN})
        mock_status.return_value = host_statuses
        rule = policies.SERVERS % 'detail'
        # server 'detail' policy is checked before unknown host_status
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts. To verify the unknown host_status
        # policy we need to disallow host_status policy for everyone.
        rule_host_status = policies.SERVERS % 'show:host_status'
        self.policy.set_rules({
            rule: "@",
            rule_host_status: "!"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.16')
        rule_name = policies.SERVERS % 'show:host_status:unknown-only'
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.all_projects_admin_authorized_contexts,
            rule_name, self.controller.detail, req,
            fatal=False)
        for resp in authorize_res:
            self.assertIn('host_status', resp['servers'][0])
            self.assertEqual(fields.HostStatus.UNKNOWN,
                resp['servers'][0]['host_status'])
        for resp in unauthorize_res:
            self.assertNotIn('host_status', resp['servers'][0])

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    @mock.patch('nova.compute.api.API.get_instance_host_status')
    def test_server_show_with_unknown_host_status_policy(self,
        mock_status, mock_block):
        mock_status.return_value = fields.HostStatus.UNKNOWN
        rule = policies.SERVERS % 'show'
        # server 'show' policy is checked before unknown host_status
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts. To verify the unknown host_status
        # policy we need to disallow host_status policy for everyone.
        rule_host_status = policies.SERVERS % 'show:host_status'
        self.policy.set_rules({
            rule: "@",
            rule_host_status: "!"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.16')
        rule_name = policies.SERVERS % 'show:host_status:unknown-only'
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.project_admin_authorized_contexts,
            rule_name, self.controller.show, req, 'fake',
            fatal=False)
        for resp in authorize_res:
            self.assertIn('host_status', resp['server'])
            self.assertEqual(
                fields.HostStatus.UNKNOWN, resp['server']['host_status'])
        for resp in unauthorize_res:
            self.assertNotIn('host_status', resp['server'])

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    @mock.patch('nova.compute.api.API.get_instance_host_status')
    @mock.patch('nova.compute.api.API.rebuild')
    def test_server_rebuild_with_unknown_host_status_policy(self, mock_rebuild,
        mock_status, mock_bdm):
        mock_status.return_value = fields.HostStatus.UNKNOWN
        rule = policies.SERVERS % 'rebuild'
        # server 'rebuild' policy is checked before unknown host_status
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts. To verify the unknown host_status
        # policy we need to disallow host_status policy for everyone.
        rule_host_status = policies.SERVERS % 'show:host_status'
        self.policy.set_rules({
            rule: "@",
            rule_host_status: "!"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.75')
        rule_name = policies.SERVERS % 'show:host_status:unknown-only'
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.project_admin_authorized_contexts,
            rule_name, self.controller._action_rebuild,
            req, self.instance.uuid,
            body={'rebuild': {"imageRef": uuids.fake_id}},
            fatal=False)
        for resp in authorize_res:
            self.assertIn('host_status', resp.obj['server'])
            self.assertEqual(
                fields.HostStatus.UNKNOWN, resp.obj['server']['host_status'])
        for resp in unauthorize_res:
            self.assertNotIn('host_status', resp.obj['server'])

    @mock.patch('nova.objects.BlockDeviceMappingList.bdms_by_instance_uuid')
    @mock.patch('nova.compute.api.API.get_instance_host_status')
    @mock.patch.object(InstanceGroup, 'get_by_instance_uuid')
    @mock.patch('nova.compute.api.API.update_instance')
    def test_server_update_with_unknown_host_status_policy(self,
        mock_update, mock_group, mock_status, mock_bdm):
        mock_update.return_value = self.instance
        mock_status.return_value = fields.HostStatus.UNKNOWN
        rule = policies.SERVERS % 'update'
        # server 'update' policy is checked before unknown host_status
        # policy so we have to allow it for everyone otherwise it will fail
        # first for unauthorized contexts. To verify the unknown host_status
        # policy we need to disallow host_status policy for everyone.
        rule_host_status = policies.SERVERS % 'show:host_status'
        self.policy.set_rules({
            rule: "@",
            rule_host_status: "!"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.75')
        rule_name = policies.SERVERS % 'show:host_status:unknown-only'
        authorize_res, unauthorize_res = self.common_policy_auth(
            self.project_admin_authorized_contexts,
            rule_name, self.controller.update,
            req, self.instance.uuid,
            body={'server': {'name': 'test'}},
            fatal=False)
        for resp in authorize_res:
            self.assertIn('host_status', resp['server'])
            self.assertEqual(
                fields.HostStatus.UNKNOWN, resp['server']['host_status'])
        for resp in unauthorize_res:
            self.assertNotIn('host_status', resp['server'])

    @mock.patch('nova.compute.api.API.create')
    def test_create_requested_destination_server_policy(self,
        mock_create):
        # 'create' policy is checked before 'create:requested_destination' so
        # we have to allow it for everyone otherwise it will
        # fail for unauthorized contexts here.
        rule = policies.SERVERS % 'create'
        self.policy.set_rules({rule: "@"}, overwrite=False)
        req = fakes.HTTPRequest.blank('', version='2.74')

        def fake_create(context, *args, **kwargs):
            for attr in ['requested_host', 'requested_hypervisor_hostname']:
                if context in self.all_projects_admin_authorized_contexts:
                    self.assertIn(attr, kwargs)
                if context not in self.all_projects_admin_authorized_contexts:
                    self.assertNotIn(attr, kwargs)
            return ([self.instance], '')
        mock_create.side_effect = fake_create

        body = {
            'server': {
                'name': 'server_test',
                'imageRef': uuids.fake_id,
                'flavorRef': uuids.fake_id,
                'networks': [{
                    'uuid': uuids.fake_id
                }],
                'host': 'fake',
                'hypervisor_hostname': 'fake'
            },
        }

        self.common_policy_auth(self.all_projects_admin_authorized_contexts,
                                self.rule_requested_destination,
                                self.controller.create,
                                req, body=body)

    @mock.patch(
        'nova.servicegroup.api.API.service_is_up',
        new=mock.Mock(return_value=True))
    @mock.patch(
        'nova.objects.Instance.image_meta',
        new=objects.ImageMeta.from_dict({}))
    @mock.patch('nova.compute.api.API._check_requested_networks')
    @mock.patch('nova.compute.api.API._allow_resize_to_same_host')
    @mock.patch('nova.objects.RequestSpec.get_by_instance_uuid')
    @mock.patch('nova.objects.Instance.save')
    @mock.patch('nova.conductor.ComputeTaskAPI.resize_instance')
    def test_cross_cell_resize_server_policy(
        self, mock_resize, mock_save, mock_rs, mock_allow, m_net
    ):

        # 'migrate' policy is checked before 'resize:cross_cell' so
        # we have to allow it for everyone otherwise it will
        # fail for unauthorized contexts here.
        rule = 'os_compute_api:os-migrate-server:migrate'
        self.policy.set_rules({rule: "@"}, overwrite=False)
        rule_name = policies.CROSS_CELL_RESIZE
        req = fakes.HTTPRequest.blank('', version='2.56')

        def fake_get(*args, **kwargs):
            inst = fake_instance.fake_instance_obj(
                self.project_member_context,
                id=1, uuid=uuids.fake_id, project_id=self.project_id,
                user_id='fake-user', vm_state=vm_states.ACTIVE,
                expected_attrs=['system_metadata', 'info_cache'],
                launched_at=timeutils.utcnow(), host='host')
            inst.services = objects.ServiceList(self.project_member_context)
            inst.services.objects.append(
                objects.Service(
                    context=self.project_member_context,
                    host=inst.host,
                    binary='nova-compute',
                    topic='compute',
                    report_count=0
                )
            )
            return inst

        self.mock_get.side_effect = fake_get

        def fake_validate(context, instance,
            host_name, allow_cross_cell_resize):
            if context in self.cross_cell_authorized_contexts:
                self.assertTrue(allow_cross_cell_resize)
            if context not in self.cross_cell_authorized_contexts:
                self.assertFalse(allow_cross_cell_resize)
            return objects.ComputeNode(host=1, hypervisor_hostname=2)

        self.stub_out(
                'nova.compute.api.API._validate_host_for_cold_migrate',
                fake_validate)

        self.common_policy_auth(self.cross_cell_authorized_contexts,
                                rule_name,
                                self.m_controller._migrate,
                                req, self.instance.uuid,
                                body={'migrate': {'host': 'fake'}},
                                fatal=False)

    def test_network_attach_external_network_policy(self):
        # NOTE(gmann): Testing policy 'network:attach_external_network'
        # which raise different error then PolicyNotAuthorized
        # if not allowed.
        neutron_api = neutron.API()
        for context in self.all_projects_admin_authorized_contexts:
            neutron_api._check_external_network_attach(context,
                    [{'id': 1, 'router:external': 'ext'}])
        unauth = (set(self.all_contexts) -
                  set(self.all_projects_admin_authorized_contexts))
        for context in unauth:
            self.assertRaises(exception.ExternalNetworkAttachForbidden,
                              neutron_api._check_external_network_attach,
                              context, [{'id': 1, 'router:external': 'ext'}])

    def test_zero_disk_flavor_policy(self):
        # NOTE(gmann): Testing policy 'create:zero_disk_flavor'
        # which raise different error then PolicyNotAuthorized
        # if not allowed.
        image = {'id': uuids.image_id, 'status': 'foo'}
        flavor = objects.Flavor(
            vcpus=1, memory_mb=512, root_gb=0, extra_specs={'hw:pmu': "true"})
        compute_api = compute.API()
        for context in self.all_projects_admin_authorized_contexts:
            compute_api._validate_flavor_image_nostatus(context,
                    image, flavor, None)
        unauth = (set(self.all_contexts) -
                  set(self.all_projects_admin_authorized_contexts))
        for context in unauth:
            self.assertRaises(
                exception.BootFromVolumeRequiredForZeroDiskFlavor,
                compute_api._validate_flavor_image_nostatus,
                context, image, flavor, None)


class ServersNoLegacyNoScopeTest(ServersPolicyTest):
    """Test Servers API policies with deprecated rules disabled, but scope
    checking still disabled.
    """
    without_deprecated_rules = True
    rules_without_deprecation = {
        policies.SERVERS % 'show:flavor-extra-specs':
            base_policy.PROJECT_READER_OR_ADMIN,
    }

    def setUp(self):
        super(ServersNoLegacyNoScopeTest, self).setUp()

        # Disabling legacy rule support means that we no longer allow
        # random roles on our project to take action on our
        # resources. Legacy admin will have access.
        self.project_action_authorized_contexts = (
            self.project_member_or_admin_with_no_scope_no_legacy)

        # The only additional role that can read our resources is our
        # own project_reader.
        self.project_reader_authorized_contexts = (
            self.project_reader_or_admin_with_no_scope_no_legacy)

        # Disabling legacy support means random roles lose power to
        # see everything in their project.
        self.reduce_set('everyone_authorized',
                        self.all_contexts - set([self.project_foo_context,
                                                 self.system_foo_context]))

        # Disabling legacy support means readers and random roles lose
        # power to create things on their own projects. Note that
        # system_admin and system_member are still here because we are
        # not rejecting them by scope, even though these operations
        # with those tokens are likely to fail because they have no
        # project.
        self.reduce_set('project_member_authorized',
                        self.all_contexts - set([
                            self.system_reader_context,
                            self.system_foo_context,
                            self.project_reader_context,
                            self.project_foo_context,
                            self.other_project_reader_context]))


class ServersScopeTypePolicyTest(ServersPolicyTest):
    """Test Servers APIs policies with system scope enabled.
    This class set the nova.conf [oslo_policy] enforce_scope to True
    so that we can switch on the scope checking on oslo policy side.
    It defines the set of context with scoped token
    which are allowed and not allowed to pass the policy checks.
    With those set of context, it will run the API operation and
    verify the expected behaviour.
    """

    def setUp(self):
        super(ServersScopeTypePolicyTest, self).setUp()
        self.flags(enforce_scope=True, group="oslo_policy")

        # These policy are project scoped only and 'create' policy is checked
        # first so even we allow it for everyone the system scoped context
        # cannot validate these as they fail on 'create' policy due to
        # scope_type. So we need to set rule name as None to skip the policy
        # error message assertion in base class. These rule name are only used
        # for error message assertion.
        self.rule_trusted_certs = None
        self.rule_attach_network = None
        self.rule_attach_volume = None
        self.rule_requested_destination = None
        self.rule_forced_host = None

        # With scope checking enabled, system admins no longer have
        # admin-granted project resource access.
        self.reduce_set('project_action_authorized',
                        set([self.legacy_admin_context,
                             self.project_admin_context,
                             self.project_member_context,
                             self.project_reader_context,
                             self.project_foo_context]))

        # No change from the base behavior here, but we need to
        # re-build this from project_action_authorized, since we
        # changed it above.
        self.project_reader_authorized_contexts = (
            self.project_action_authorized_contexts)

        # With scope checking enabled, system users no longer have
        # project access, even to create their own resources.
        self.reduce_set('project_member_authorized', self.all_project_contexts)

        # With scope checking enabled, system admin is no longer an
        # admin of project resources.
        self.reduce_set('project_admin_authorized',
                        set([self.legacy_admin_context,
                             self.project_admin_context]))
        self.reduce_set('all_projects_admin_authorized',
                        set([self.legacy_admin_context,
                             self.project_admin_context]))

        # With scope checking enabled, system users also lose access to read
        # project resources.
        self.reduce_set('everyone_authorized',
                        self.all_contexts - self.all_system_contexts)


class ServersNoLegacyPolicyTest(ServersScopeTypePolicyTest):
    """Test Servers APIs policies with system scope enabled,
    and no more deprecated rules.
    """
    without_deprecated_rules = True
    rules_without_deprecation = {
        policies.SERVERS % 'show:flavor-extra-specs':
            base_policy.PROJECT_READER_OR_ADMIN,
    }

    def setUp(self):
        super(ServersNoLegacyPolicyTest, self).setUp()

        # Disabling legacy support means legacy_admin is no longer
        # powerful on our project. Also, we drop the "any role on the
        # project means you can do stuff" behavior, so project_reader
        # and project_foo lose power.
        self.project_action_authorized_contexts = (
            self.project_member_or_admin_with_scope_no_legacy)

        # Only project_reader has additional read access to our
        # project resources.
        self.project_reader_authorized_contexts = (
            self.project_action_authorized_contexts |
            set([self.project_reader_context]))

        # Disabling legacy support means random roles lose power to
        # see everything in their project.
        self.reduce_set(
            'everyone_authorized',
            self.all_project_contexts - set([self.project_foo_context]))

        # Disabling legacy support means readers and random roles lose
        # power to create things on their own projects.
        self.reduce_set('project_member_authorized',
                        self.all_project_contexts - set([
                            self.project_foo_context,
                            self.project_reader_context,
                            self.other_project_reader_context,
                        ]))
